--[[
Temp-Page Key - INSTALLER
Created by Jason Giaffo
Contact: http://GiaffoDesigns.com/contact/
Last updated July 11, 2017

Version 1.0.2
Tested through GrandMA2 v3.3.2.2
--]]




---- User Config ----

local userconfig = {}

---------------------------------------------------------------------------------
---- The default settings will install fine. But if you have specific wishes ----
----- for where things will be stored, you can change those settings below. -----
---------------------------------------------------------------------------------

userconfig.fixture_num = 50001                   --fixture number to be used by installer for temp page key. If taken, will advance to next available

userconfig.executor = {page = 1 , exec = 201}    --executor number where first temp key will be stored. 
userconfig.dmx_addr = {univ = 200, ch = 1}       --first dmx address that will be tried for installation. Will advance to nearest slot open for both patch and DMX removes. 2 DMX channels used total.

userconfig.plugin_start = 500                    --position where first generated plugin (of 3 total) will be installed.
userconfig.seq_num = 1000                        --sequence number where remote sequence will be stored









----------------------------------------------------------------------------------------------------------
-------------------------------------- DO NOT EDIT BELOW THIS POINT --------------------------------------
----------------------------------------------------------------------------------------------------------

---- FUNCTIONS ----
local unpack = table.unpack



local function advanceSpace(poolType, start, length, pad_before, pad_after)
	--new version, has checkSpace() function embedded, padding added
	--padding parameters are optional; default to 0 if not provided
	local getHandle = gma.show.getobj.handle
	local getVerify = function(obj)
		gma.echo('ignore potential "error: returns bool:result" message below')
		return gma.show.getobj.verify(getHandle(obj))
	end
	
	local function checkSpace(poolType, start, length) --checks if range of pool spaces is empty
	  local finish = start + length - 1 --set our finishing point
	  local emptyStatus = true
	  for i = start, finish do
		if getVerify(poolType..' '..tostring(i)) then --if space is not empty
		  emptyStatus = false
		  break
		end
	  end
	  return emptyStatus
	end

	local pad_before	= pad_before	or 0							--set for 0 if not provided
	local pad_after		= pad_after		or 0							--set for 0 if not provided
	pad_before, pad_after = math.abs(pad_before), math.abs(pad_after)
	local length_actual = length + pad_before + pad_after				--length that will be used in space check
	
	local finalStart = start
	while checkSpace(poolType, finalStart, length_actual) == false do
		finalStart = finalStart + 1
	end
  
	finalStart = finalStart + pad_before								--offset returned answer for pre-padding
	return finalStart
end


function setRange(range, startPoint, loops) --for assigning the range of locations for generated pool items
  if startPoint then setRange_ct = startPoint end
  if (not setRange_ct) then setRange_ct = 1 end

  if (not loops) then
	local start = setRange_ct
	local finish = setRange_ct + range - 1
	setRange_ct = setRange_ct + range
	return start, finish
	
  elseif loops then
	local results = {}
    for i = 1, loops do
	  results[i] = setRange_ct
	  setRange_ct = setRange_ct + range
	end
	return unpack(results)
  end
end


local function advanceExec(page, exec, ct, minExec)
  --check each spot from start until end
  --if there is a collision, function starts over at next available executor on page
  --if the executor number hits 100, 199, or 210, and there still are more to go, then
  --the count restarts on the next page at the minimum executor number.
  --if there is not enough space to pull this off on any page, the function returns nil and an error message
	local getHandle = gma.show.getobj.handle
	local function getClass(obj)
		return gma.show.getobj.class(getHandle(obj))
	end
	
	local function getExecutorStatus(page, executor)  
	  local slotStatus = getHandle('Executor '..page..'.'..executor); --determine if slot is empty
				
	  if slotStatus then                                --return value based on executor handle
		if getHandle('Executor '..page..'.'..executor..' Cue') then
			local c = getClass('Executor '..page..'.'..executor..' Cue');
			if c == "CMD_SEQUENCE" then
			  return 'OFF' --response for non-active executor
			elseif c == "CMD_CUE" then
			  return 'ON' --response for active executor
			end
		else
			return 'NON-SEQ' --response for non-sequence executors
		end
	  else
		return 'EMPTY' --response for empty executor slots
	  end
	end
	
  local function checkPass(exec, ct)
    local limits = {100, 199, 210}
	local status_pass = true
	for i = 1, #limits do
	  if exec <= limits[i] and (exec+ct-1) > limits[i] then 
		status_pass = false 
		break
	  end	  
	end
	
	return status_pass
  end
  
  local function checkNum(num)
    local limits = {100, 199, 210}
	local status_pass = true
	for i = 1, #limits do
	  if num == limits[i] then status_pass = false; break; end;
	end
	
	return status_pass
  end

  local error_msg = 'Error: minExec set too high'
  
  -- set minimum restart executor for a new page
  local minExec = minExec
  if not minExec then 
	local starts = {1, 101, 201}						--base off location of provided desired executor position
	for i = 1, #starts do
		if exec > starts[i] then minExec = starts[i] end
	end
  end
  if not ct then ct = 1 end
  
  local addPage = true
  if not checkPass(minExec, ct) then addPage = false end
  
  local pageCurrent = page
  local execCurrent = exec
  local pass = false

  while pass == false do
    if checkPass(execCurrent, ct) then
		for i = 1, ct do
		  if getExecutorStatus(pageCurrent, execCurrent+i-1) == 'EMPTY' then
			if i == ct then pass = true end
		  else
			if checkNum(execCurrent+i) then
				execCurrent = execCurrent + i
				break
			else
				if addPage then
					pageCurrent = pageCurrent + 1
					execCurrent = minExec
					break
				else
					return nil, error_msg
				end
			end
		  end
		end
	else
		if addPage then
			pageCurrent = pageCurrent + 1
			execCurrent = minExec
		else
			return nil, error_msg
		end
	end
  end
  
  
  return {page = pageCurrent, exec = execCurrent, str = tostring(pageCurrent)..'.'..tostring(execCurrent)}
end


local function createPlugin(num, name, script, EOL)

  -- establish if plugin will Execute On Load --
  local EOLnum
  if EOL then EOLnum = 1 
  else EOLnum = 0 end
  
  -- Establish filepath --
  local plugin = {}
  plugin.name = 'tempFile_createPlugin'
  plugin.directory = gma.show.getvar('PLUGINPATH')..'/'
  plugin.fullpathLUA = plugin.directory..plugin.name..'.lua'
  plugin.fullpathXML = plugin.directory..plugin.name..'.xml'


  -- Create text for plugin XML file --
  local xmlText = [[
<?xml version="1.0" encoding="utf-8"?>
<MA xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns="http://schemas.malighting.de/grandma2/xml/MA" xsi:schemaLocation="http://schemas.malighting.de/grandma2/xml/MA http://schemas.malighting.de/grandma2/xml/3.2.2/MA.xsd" major_vers="3" minor_vers="2" stream_vers="2">
	<Info datetime="2016-09-26T20:40:54" showfile="dummyfile" />
	<Plugin index="1" execute_on_load="]]..EOLnum..[[" name="]]..name..[[" luafile="]]..plugin.name..[[.lua" />
</MA>]]

  -- Write XML file to disk --
  local fileXML = assert(io.open(plugin.fullpathXML, 'w'))
  fileXML:write(xmlText)
  fileXML:close()
  
  -- Write LUA file to disk --
  local fileLUA = assert(io.open(plugin.fullpathLUA, 'w'))
  fileLUA:write(script)
  fileLUA:close()

  -- import plugin --
  gma.cmd('SelectDrive 1') --select the internal drive as default read path
  gma.cmd('Import \"'..plugin.name..'\" Plugin '..tostring(num)) --load new plugin into showfile

  -- delete temp files --
  os.remove(plugin.fullpathXML)
  os.remove(plugin.fullpathLUA)
end


local function exec_props_string(str)
	local props = {}
	local indices = {1, 1}

	while indices[2] < #str do
	  indices = {str:find('%s[^=]+[=][^,]+', indices[2])}
	  if (not indices[1]) then break end
	  indices[1] = indices[1] + 1

	  local t = str:sub(unpack(indices))
	  local key = t:sub(t:find('[^=]+'))

	  local indices2 = {t:find('[=][^=]+')}
	  indices2[1] = indices2[1] + 1
	  local value = t:sub(unpack(indices2))

	  props[key] = value
	end

	local str_props = ''
	for k, v in pairs(props) do
	  if v:find('%s') then v = '\"'..v..'\"' end
	  str_props = str_props..[[ /]]..k..[[=]]..v
	end

	return str_props
end


local csv = { --module beta version
	objToArray = function(obj, pBar)

	-------- NESTED FUNCTIONS --------

		---- converts a CSV line to a number-indexed array ----
		local function CSVlineToArray(str)
		  local t = {}
		  local indices = {1, 1}
		  local remaining = true
		  local restart = 1
		  while remaining == true do
			local pt = {str:find(',', restart)}		--find next comma at or after restart point
			if pt[1] then
			  t[#t+1] = str:sub(restart, pt[1]-1)	--extract string from restart point to one position before comma
			  restart = pt[2] + 1 					--reset restart point for one position after most recent found comma
			else
			  remaining = false
			  t[#t+1] = str:sub(restart, str[-1])
			end
		  end

		  return t
		end


		---- Converts a CSV line to an array using the indices of an input header table ----
		local function CSVsubheaderToArray(str, header)
		  local t = {}
		  local indices = {1, 1}
		  local remaining = true
		  local restart = 1
		  local index = 1
		  while remaining == true do
			local pt = {str:find(',', restart)}		--find next comma at or after restart point
			if pt[1] and index < #header then
			  t[header[index]] = str:sub(restart, pt[1]-1)	--extract string from restart point to one position before comma
			  restart = pt[2] + 1 					--reset restart point for one position after most recent found comma
			  index = index + 1
			else
			  remaining = false
			  t[header[index]] = str:sub(restart, str[-1])
			end
		  end

		  return t
		end

		---- takes a CSV file, uses the first row as the headers , and indexes the contents into a table using those headers 
		local function CSVfileToArray(file, pBar)	
			local header = {}
			local results = {}

			local firstLine = true
			for line in io.lines(file) do
			  if firstLine then
				header = CSVlineToArray(line)
				for i, v in ipairs(header) do
					if v == '' then header[i] = tostring(i) end
					header[i] = v:lower()
					for j = 1, (i-1) do
						if header[i] == header[j] then header[i] = header[i]..tostring(i) end
					end
				end
				firstLine = false
			  else
				results[#results+1] = CSVsubheaderToArray(line, header)
				if pBar then pBar:set(1, true) end
			  end
			end

			return results
		  
		end
		
		
		
		---- main function ----
		-- filepaths
		local dir	= {temp = gma.show.getvar('PATH')..'/reports/'}
		local fn 	= {temp = 'export_csvobject.csv'}
		local file	= {temp = dir.temp..fn.temp}
		
		-- export object
		gma.cmd('SelectDrive 1')
		if obj then	gma.cmd('List '..obj..' /f=\"'..fn.temp..'\"')
		else gma.cmd('List /f=\"'..fn.temp..'\"') end
		
		local t		--declare before if statement
		-- check if file exists; if not, no object was found for export
		local test = io.open(file.temp)
		if test then									--if file is present
			test:close()								--close test file
			
			-- convert into table
			t = CSVfileToArray(file.temp, pBar)
				
			-- delete temp file
			os.remove(file.temp)
		else
			t = {}
		end

		
		-- return result
		return t
	end,
	

	
	---- function to take a two-dimensional table and convert it to a CSV text string
	write_table = function(list)
		-- nested function to write a single line
		local function write_line(list)
			local t = list[1]
			if #list > 1 then
			  for i = 2, #list do
				t = t..','..list[i]
			  end
		    end
		  
			if t == nil then t = '' end
		  
			return t
		end
	
		-- main function
		local t = ''
		for i = 1, #list do
		  t = t..write_line(list[i])..'\n'
		end

		return t
	end
}


local gui = {
-- cmdview() requires advanceSpace function
	superBar = {
		value = 0,
		
		new = function(self, name)
			o = {handle = gma.gui.progress.start(name)}

			setmetatable(o, self)
			self.__index = self
			return o
		end,
		
		set = function(self, num, add)
			if add then num = self.value + num end
			
			num = math.floor(num)
			gma.gui.progress.set(self.handle, num)
			self.value = num
		end,
		
		setrange = function(self, bottom, top)
			if not bottom or not top then return nil end
			gma.gui.progress.setrange(self.handle, bottom, top)
			self.range = {top = top, bottom = bottom}
		end,
		
		settext = function(self, text)
			gma.gui.progress.settext(self.handle, text)
		end,
		
		rename = function(self, name)
			gma.gui.progress.stop(self.handle)
			self.handle = gma.gui.progress.start(name)
			if self.range then gma.gui.progress.setrange(self.handle, self.range.bottom, self.range.top) end
		end,
		
		stop = function(self)
			gma.gui.progress.stop(self.handle)
		end,
		
		time_move = function(self, target, time, add, ignore_top)
			if add then target = self.value + target end
			local start = self.value
			local finish = math.floor(target)
			
			local sleepPeriod
			if math.abs(time) ~= time then sleepPeriod = (1 / time)											--allow for negative time values to trigger a "rate" function
			else sleepPeriod = time / (math.abs(finish - start)) end
			
			if not ignore_top and self.range and finish > self.range.top then finish = self.range.top end	--set finish point to top of progress-bar range unless specified to ignore
			
			local dir = 1
			if start > finish then dir = -1 end
			
			for i = start, finish, dir do
				self:set(i)
				gma.sleep(sleepPeriod)
			end	
		end
	}, --end of superBar object
	
	
	msgbox = function(title, message_box, message_text)
	  -- v. 1.0
	  -- function avoids using confirmation box function with version 3.1.2.5, where it doesn't exist
	  local confirm_method
	  
	  local version = gma.show.getvar('version')
	  
	  if version:find('3.1.2') == 1 then 
		confirm_method = 'textinput'
	  else 
		confirm_method = 'box' end
	  
	  if confirm_method == 'box' then
		gma.gui.msgbox(title, message_box)
	  elseif confirm_method == 'textinput' then
		gma.textinput(title, message_text)
	  end
	end,
	
	
	confirm = function(title, message_box, message_text)
	  -- v. 1.0
	  -- function avoids using confirmation box function with version 3.1.2.5, which crashes the software
	  local confirm_method
	  
	  local version = gma.show.getvar('version')
	  
	  if version:find('3.1.2') == 1 then 
		confirm_method = 'textinput'
	  else 
		confirm_method = 'box' end
	  
	  if confirm_method == 'box' then
		return gma.gui.confirm(title, message_box)
	  elseif confirm_method == 'textinput' then
		local t = gma.textinput(title, message_text)
		if t then t = true end
		return t
	  end
	end,
	
	
	print = function(str)
		gma.echo(str)
		gma.feedback(str)
	end,
	
	
	cmdview = function(screen)
		if not screen then screen = 2 end
		local cmd_script = [[
<?xml version="1.0" encoding="utf-8"?>
<MA xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns="http://schemas.malighting.de/grandma2/xml/MA" xsi:schemaLocation="http://schemas.malighting.de/grandma2/xml/MA http://schemas.malighting.de/grandma2/xml/3.2.2/MA.xsd" major_vers="3" minor_vers="2" stream_vers="2">
	<Info datetime="2017-06-16T16:29:12" showfile="temp key install - pre" />
	<View index="31" name="Sys" display_mask="8">
		<BitMap width="96" height="48">
			<Image>
				dBUY/09RYP95Cgz/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+DBwf/
				gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ACAv9+BAT/
				cBET/09RYP95Cgz/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/
				gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4MGBv+BAwP/
				gQIC/4ECAv+BAgL/gQIC/4EDA/+DBwf/gQIC/4ECAv+CBAT/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/
				gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/
				gQIC/4ECAv+BAgL/gAIC/4ACAv9/BAT/RlRk/wSryf9NKDD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/
				fwAA/38AAP9/AAD/fwAA/38AAP+yZWX/qVVV/7hycv+pVVX/s2ho/4UMDP9/AAD/fwAA/38AAP9/AAD/
				fwAA/38AAP9/AAD/fwAA/38AAP+BAgL/SEhU/wSryf9NKDD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/
				fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/
				fwAA/38AAP9/AAD/fwAA/6dRUf+vYWH/pUtL/6tZWf+uXl7/tW1t/5YuLv/Gjo7/qlVV/6pVVf+2cHD/
				qFJS/6VMTP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/
				fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP+BAgL/cQ4Q/0YnLv92Bwf/
				gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+DBwf/gwYG/4QICP+ECAj/
				hAkJ/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ACAv9+BAT/bQ0O/0YnLv92Bwf/
				gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/
				gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4MHB/+MFxf/gwYG/4MFBf+ECAj/
				hAgI/4IEBP+DBgb/gwYG/4MHB/+DBwf/hAgI/4IFBf+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/
				gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/
				gAIC/4ACAv9/BAT/EzxL/yE7RP8RKzX/GjQ+/wwlL/8QKjT/FS84/xUvOf8TLTb/BiAp/wYgKf8GICn/
				BiAp/wYgKf8GICn/BiAp/wYgKf8GICn/BiAp/wYgKf8GICn/BiAp/wYgKf8GICn/BiAp/wYgKf8JMUH/
				CR4l/wQSGP8GHCX/FDlH/yA7RP8pQ03/HDY//x85Qv8cNj//FC44/yA6RP8fOUL/Ey02/yA6RP8WMDn/
				HzlC/xkzPP8ULjj/CyUv/wslL/8gOkT/FjA5/xkzPP8ZMzz/JT9I/yI8Rf8WMDn/HTdB/yI8Rf8OKDL/
				FzE7/xkzPP8gOkT/FjA5/xo0Pv8NJzD/HTdB/xcxO/8RKzX/FzE7/w4oMv8TLTb/ECoz/x85Qv8jPUf/
				ESs1/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/
				CiQt/wokLf8KJC3/CiMt/wkkLv8OMT7/CCk2/wciLP8LMkH/FzhF/yUyN/8mMzj/KDU6/w8bIP8hLjP/
				LTk+/zE+Q/8TICX/AA0R/wANEf8ADRH/AA0R/wANEf8ADRH/AA0R/wANEf8ADRH/AA0R/wANEf8ADRH/
				AA0R/wMOEv8ADRH/AA0R/wANEf8EJjL/Bh4n/wJcgP8CKDb/GjQ+/ysrK/8zMzP/MTEx/zExMf8iIiL/
				Hx8f/yoqKv8iIiL/DQ0N/y4uLv8bGxv/Kioq/x4eHv8uLi7/BgYG/wYGBv8oKCj/IiIi/xwcHP8ZGRn/
				LS0t/ysrK/8fHx//HBwc/y0tLf8PDw//FhYW/ycnJ/8eHh7/Hh4e/x8fH/8PDw//Jycn/yoqKv8VFRX/
				GRkZ/wkJCf8MDAz/Dw8P/x8fH/8kJCT/EhIS/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8IISv/ACMx/wFObP8GIy7/
				BBEV/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/xMLA/8AAAD/AAAA/wAAAP8GExf/CSIr/wQSGP8FISz/
				FjA5/ygoKP8wMDD/LS0t/zExMf8bGxv/Gxsb/yQkJP8oKCj/Dw8P/zY2Nv8fHx//JCQk/yIiIv8iIiL/
				BgYG/wYGBv8kJCT/IiIi/xISEv8GBgb/Kioq/yQkJP8eHh7/Kysr/xgYGP8kJCT/MTEx/yUlJf8TExP/
				GRkZ/wAAAP8SEhL/Hx8f/yUlJf8nJyf/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8IIiv/ACAs/wATG/8GJzP/BBEV/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/xMLA/8AAAD/
				AAAA/wAAAP8GExf/GCUp/xoaGv8YJSn/HDY//y4uLv83Nzf/NDQ0/zc3N/8lJSX/Hh4e/y4uLv8zMzP/
				AAAA/wAAAP8xMTH/Ojo6/xYWFv8GBgb/ExMT/zk5Of8eHh7/NDQ0/zQ0NP8KCgr/HBwc/ygoKP8rKyv/
				IiIi/y4uLv8KCgr/NjY2/ycnJ/8cHBz/Hx8f/w0NDf8GBgb/FhYW/ygoKP80NDT/FhYW/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8IIiv/Fi43/xglKf8VNUH/BCMW/wATAP8AEwD/
				ABMA/wATAP8AEwD/ABMA/wATAP8AEwD/ABMA/wATAP8AEwD/ABMA/wATAP8AEwD/ABMA/wATAP8AEwD/
				ABMA/wATAP8AEwD/ABMA/xAaA/8AAAD/AAAA/wAAAP8FEhf/GCUp/xoaGv8YJSn/GjQ+/zAwMP85OTn/
				NDQ0/zc3N/8lJSX/Hh4e/y4uLv8zMzP/EBAQ/zc3N/8hISH/Li4u/x8fH/8tLS3/BgYG/wYGBv8tLS3/
				JSUl/xYWFv8hISH/OTk5/zk5Of8TExP/Jycn/zo6Ov8NDQ3/ISEh/zAwMP8tLS3/JSUl/ysrK/8KCgr/
				MDAw/yUlJf8VFRX/IiIi/wwMDP8TExP/ExMT/zQ0NP8zMzP/FhYW/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8IIiv/
				GCUp/xoaGv8WMDn/Gk8t/ysrK/8nJyf/MTEx/yQkJP8hISH/IiIi/w8PD/8bGxv/JCQk/yMjI/8WFhb/
				Dg4O/ykpKf8pKSn/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8EERX/
				GCUp/xoaGv8YJSn/GjQ+/zAwMP85OTn/NDQ0/zY2Nv8zMzP/Hx8f/zY2Nv8xMTH/AAAA/wAAAP8xMTH/
				Ojo6/xYWFv8GBgb/DQ0N/ycnJ/8rKyv/Kioq/x8fH/8JCQn/EBAQ/zo6Ov85OTn/GBgY/zQ0NP8fHx//
				ExMT/xwcHP8TExP/PT09/xMTE/8qKir/Ojo6/ycnJ/8TExP/BgYG/xsbG/8nJyf/Kioq/zQ0NP8kJCT/
				Jycn/yIiIv8VFRX/BgYG/zMzM/8SEhL/JCQk/xsbG/8uLi7/LS0t/ycnJ/8lJSX/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8IIiv/GCUp/xoaGv8WMDn/EUI4/xQhJv8YJSr/FSIn/xQhJf8PHCH/
				EyAl/w4bIP8WIyj/FSIn/xMgJf8NGh//Cxgd/w4bH/8SHiP/AA0R/wANEf8ADRH/AA0R/wANEf8ADRH/
				AA0R/wANEf8ADRH/AA0R/wANEf8EICv/GCUp/xoaGv8YJSn/GTM8/y0tLf8zMzP/LS0t/zc3N/8oKCj/
				FRUV/x4eHv8rKyv/DQ0N/ysrK/8ZGRn/MTEx/x8fH/8eHh7/BAQE/w8PD/8rKyv/Jycn/ysrK/8bGxv/
				Kioq/yEhIf8wMDD/Hx8f/zExMf8fHx//LS0t/yEhIf8uLi7/ISEh/ysrK/8kJCT/Kysr/yEhIf8oKCj/
				JSUl/ycnJ/8uLi7/Kioq/y4uLv8tLS3/MDAw/x8fH/8tLS3/Hx8f/zQ0NP8cHBz/MTEx/yEhIf8wMDD/
				ISEh/y0tLf8cHBz/Jycn/xsbG/8nJyf/Gxsb/xISEv8AAAD/AAAA/wAAAP8IIiv/GCUp/xoaGv8WMDn/
				HTpF/z09Pf86Ojr/Li4u/zQ0NP8jIyP/JSUl/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8EISv/CSEq/wQRFf8FHyn/
				GTM8/y0tLf82Njb/MDAw/zo6Ov8rKyv/GBgY/xsbG/8qKir/Dw8P/y0tLf8eHh7/KCgo/yQkJP8kJCT/
				BgYG/wYGBv8TExP/ExMT/xwcHP8hISH/HBwc/xsbG/8PDw//Hx8f/x8fH/8cHBz/BwcH/wwMDP8fHx//
				HBwc/xISEv8VFRX/FhYW/xgYGP8KCgr/HBwc/x8fH/8cHBz/Hx8f/xwcHP8WFhb/IiIi/xUVFf8bGxv/
				Gxsb/xMTE/8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8IIiv/CSQu/woXG/8MMD7/BCsk/wAvEf8ASRH/ADwR/wASEf8ADRH/AA0R/wANEf8ADRH/
				AA0R/wANEf8ADRH/AA0R/wAQEf8ARhH/ADUR/wANEf8ADRH/AA0R/wANEf8ADRH/AA0R/wANEf8ADRH/
				AA0R/wANEf8GHSb/CiIr/wYTF/8HISv/GjQ+/ysrK/8tLS3/MDAw/zMzM/8rKyv/GBgY/xsbG/8rKyv/
				Dw8P/zAwMP8cHBz/JSUl/yQkJP8kJCT/BAQE/wQEBP8YGBj/JCQk/yUlJf8qKir/Jycn/ygoKP8lJSX/
				Jycn/zQ0NP8tLS3/JCQk/yEhIf80NDT/Hx8f/yUlJf8nJyf/MTEx/ycnJ/8hISH/Li4u/zExMf8uLi7/
				MTEx/y4uLv8rKyv/MzMz/yUlJf8xMTH/Hx8f/ycnJ/8NDQ3/FRUV/w0NDf8VFRX/DQ0N/xUVFf8NDQ3/
				ExMT/w0NDf8TExP/DQ0N/wkJCf8AAAD/AAAA/wAAAP8IIiv/ABsl/wANEf8GKjj/BCkV/wAkAP8AIgD/
				ABwA/wAFAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAEAP8ALgD/ACkA/wADAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8GExf/GCUp/xoaGv8YJSn/HDY//ysrK/8zMzP/
				MTEx/zY2Nv8tLS3/GBgY/xgYGP8qKir/CQkJ/ygoKP8nJyf/LS0t/ygoKP8WFhb/CQkJ/w0NDf8VFRX/
				Hh4e/xMTE/8NDQ3/GBgY/xsbG/8fHx//ExMT/x8fH/8WFhb/HBwc/xYWFv8bGxv/HBwc/xkZGf8cHBz/
				JCQk/xkZGf8cHBz/EhIS/xsbG/8VFRX/EhIS/xISEv8YGBj/FRUV/w0NDf8eHh7/EBAQ/xYWFv8NDQ3/
				Gxsb/w8PD/8oKCj/EBAQ/xUVFf8NDQ3/ExMT/w0NDf8TExP/DQ0N/wkJCf8AAAD/AAAA/wAAAP8IIiv/
				ACUz/wAYIf8GLDv/BDEV/wBDAP8AMwD/ACEA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAEAP8AUgD/ADoA/wAhAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8GExf/
				FSw1/xYjJ/8ULTf/FjA5/ycnJ/8uLi7/LS0t/zc3N/8oKCj/FRUV/yIiIv8qKir/AAAA/wAAAP8oKCj/
				Li4u/wwMDP8GBgb/DAwM/yEhIf8qKir/ISEh/xgYGP8EBAT/DQ0N/zMzM/8nJyf/FRUV/ysrK/8YGBj/
				DQ0N/xYWFv8PDw//Nzc3/w8PD/8fHx//MDAw/x8fH/8MDAz/BgYG/xkZGf8iIiL/Hx8f/xwcHP8YGBj/
				Kioq/yUlJf8uLi7/ISEh/yUlJf8iIiL/EhIS/wcHB/8tLS3/DAwM/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8IISv/ACg4/wJUdP8GJDD/BRIX/wAEAP8AAQD/AAQA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAwD/AAMA/wAEAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8GExf/Bh0m/wAUHP8CHyv/DjE//wciLP8IISv/CCIr/wgiK/8IIiv/
				CCIr/wgiK/8IIiv/CCIr/wgiK/8IIiv/CCIr/wgiK/8IIiv/CCIr/wgiK/8IIiv/CCIr/wgiK/8IIiv/
				CCIr/wgiK/8IIiv/CCIr/wgiK/8IIiv/CCIr/wgiK/8IIiv/CCIr/wgiK/8IIiv/CCIr/wgiK/8IIiv/
				CCIr/wgiK/8IIiv/CCIr/wgiK/8IIiv/CCIr/wgiK/8IIiv/CCIr/wgiK/8IIiv/CCIr/wgiK/8IIiv/
				CCIr/wgiK/8IIiv/CCIr/wgiK/8IIiv/CCIr/wgiK/8IIiv/CCEr/wciLP8NLTn/CB4m/wgVGf8MKjb/
				Cxgd/wgICP8ICAj/CAgI/wgICP8ICAj/CAgI/wgICP8ICAj/CAgI/wgICP8ICAj/CAgI/wgICP8ICAj/
				CAgI/wgICP8ICAj/CAgI/wgICP8ICAj/CAgI/wgICP8ICAj/CAgI/wgICP8LGB3/Bhkh/wFRcP8BIS7/
				CCs5/wAoOP8AJTP/ACY0/wANEf8ADRH/AA0R/wAOE/8SLTf/GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/
				GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/
				GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/
				GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/
				ACUz/wAoOP8IHib/Ghoa/xoaGv8WJiz/GCUp/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/
				Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/
				Ghoa/xoaGv8YJSn/Dh8m/wgaIf8HMkL/ByYy/wJUdP8AGCH/ABoj/wAAAP8AAAD/AAAA/wAAAP8UISX/
				Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/
				Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/
				Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/
				Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/ABgh/wJUdP8IFRn/Ghoa/xgfIf8TNEH/Fi02/xgkKf8YJSn/
				GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/
				GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/GCUp/xgkKf8WLTb/GB8h/xMyPv8TPU3/CzdI/wYkMP8GLDv/
				BjFB/wYgKf8GICn/BiAp/wYhK/8SNkP/FjA5/xYwOf8WMDn/FjA5/xYwOf8WMDn/FjA5/xYwOf8WMDn/
				FjA5/xYwOf8WMDn/FjA5/xYwOf8WMDn/FjA5/xYwOf8WMDn/FjA5/xYwOf8WMDn/FjA5/xYwOf8WMDn/
				FjA5/xYwOf8WMDn/FjA5/xYwOf8WMDn/FjA5/xYwOf8WMDn/FjA5/xYwOf8WMDn/FjA5/xYwOf8WMDn/
				FjA5/xYwOf8WMDn/FjA5/xYwOf8WMDn/FjA5/xYwOf8WMDn/FjA5/xYwOf8WMDn/Biw7/wYkMP8MKjb/
				FiYs/xM0Qf8SQVP/dBUY/09RYP95Cgz/gAIC/4ECAv+BAgL/ggMD/4ECAv+BAgL/gQIC/4ECAv+BAgL/
				ggMD/4ECAv+BAgL/gQIC/4ECAv+BAgL/ggMD/4ECAv+BAgL/gQIC/4ECAv+BAgL/ggMD/4ECAv+BAgL/
				gQIC/4ECAv+BAgL/ggMD/4ECAv+BAgL/gQIC/4ECAv+BAgL/ggMD/4ECAv+BAgL/gQIC/4MGBv+BAgL/
				ggMD/4ECAv+BAgL/gQIC/4ECAv+CBAT/hQgI/4ECAv+BAgL/gwYG/4IFBf+BAgL/ggQE/4MHB/+BAgL/
				gQIC/4IEBP+BAgL/ggMD/4ECAv+BAgL/gQIC/4ECAv+BAgL/ggMD/4ECAv+BAgL/gQIC/4ECAv+BAgL/
				ggMD/4ECAv+BAgL/gQIC/4ECAv+BAgL/ggMD/4ECAv+BAgL/gQIC/4ECAv+BAgL/ggMD/4ECAv+BAgL/
				gQIC/4ECAv+BAgL/ggMD/4ECAv+AAgL/gAIC/4ACAv9/BAT/RlRk/wSryf9NKDD/fwAA/38AAP9/AAD/
				fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/
				fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/
				fwAA/38AAP9/AAD/fwAA/6ZPT/+pVFT/rFpa/6lVVf+rWFj/qFNT/6pWVv+zaWn/smdn/6tXV/+oUlL/
				n0BA/6dQUP+tXFz/rFtb/8KFhf+uXl7/rFlZ/7JlZf+ECQn/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/
				fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/
				fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP+BAgL/
				cQ4Q/0YnLv92Bwf/gAIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/
				gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/
				gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4MGBv+DBgb/gwYG/4MGBv+DBwf/
				gwcH/4MHB/+DBwf/gwYG/4MGBv+DBgb/ggQE/4IFBf+DBwf/gwcH/4QJCf+DBwf/hAgI/4MHB/+CBAT/
				gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/
				gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/
				gQIC/4ECAv+AAgL/gAIC/4ACAv9/BAT/DTRD/wkkLv8JJC3/CSMt/wokLf8KJC3/CiQt/wokLf8KJC3/
				CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/
				CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/
				CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/
				CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/
				CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/
				CiQt/wokLf8KJC3/CiQt/wokLf8KIy3/CSQt/wkkLv8NMT7/CCo2/wciLP8LM0H/Dykz/w0NDf8KCgr/
				DAwM/wkJCf8KCgr/BgYG/woKCv8JCQn/AAAA/wAAAP8DAwP/AAAA/wAAAP8EBAT/AQEB/wEBAf8AAAD/
				AAAA/wMDA/8DAwP/AAAA/wAAAP8DAwP/AAAA/wAAAP8DAwP/AwMD/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8IISv/
				ACMx/wFObP8GIy7/FzE6/ygoKP82Njb/NDQ0/zc3N/8oKCj/FRUV/y0tLf8rKyv/AAAA/wYGBv8eHh7/
				MTEx/zY2Nv8tLS3/EhIS/ycnJ/8NDQ3/JCQk/w8PD/8kJCT/Kysr/wQEBP8ZGRn/Li4u/zQ0NP8ZGRn/
				BAQE/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8IIiv/ACAs/wATG/8GJzP/HTdA/y4uLv85OTn/PDw8/z8/P/8wMDD/
				JSUl/zo6Ov80NDT/AAAA/wYGBv8fHx//MTEx/zY2Nv8tLS3/EBAQ/yUlJf8NDQ3/JCQk/xAQEP8lJSX/
				Kysr/wQEBP8WFhb/MTEx/zY2Nv8cHBz/Kioq/xsbG/8tLS3/Jycn/yIiIv8GBgb/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8IIiv/Fi43/xglKf8VNUH/
				HTdA/zAwMP86Ojr/PDw8/z8/P/8wMDD/KCgo/zExMf8zMzP/AAAA/wYGBv8fHx//Li4u/zMzM/8uLi7/
				EhIS/ycnJ/8NDQ3/JCQk/xAQEP8lJSX/Kysr/wQEBP8WFhb/MzMz/zc3N/8cHBz/Kysr/xsbG/8tLS3/
				Jycn/yIiIv8GBgb/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8IIiv/GCUp/xoaGv8WMDn/HTdA/zAwMP86Ojr/MDAw/z09Pf8kJCT/Jycn/zMzM/8zMzP/
				AAAA/wYGBv8tLS3/LS0t/ygoKP8kJCT/IiIi/ygoKP8GBgb/AAAA/wA2AP8AIQD/ACsA/wAKAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8IIiv/GCUp/xoaGv8WMDn/GzU//y0tLf82Njb/
				Li4u/zExMf8cHBz/IiIi/ysrK/8nJyf/AAAA/wQEBP8qKir/IiIi/x4eHv8hISH/Hx8f/yoqKv8EBAT/
				AAAA/wAxAP8AHgD/ACsA/wAJAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8IIiv/
				GCUp/xoaGv8WMDn/GzU//y0tLf80NDT/Nzc3/zc3N/8kJCT/JCQk/yUlJf8tLS3/AAAA/wYGBv8rKyv/
				Hh4e/x8fH/8hISH/Hx8f/ysrK/8GBgb/AAAA/wAuAP8AHAD/ACsA/wAJAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8IIiv/GCUp/xoaGv8WMDn/HTdA/ysrK/8xMTH/MTEx/zQ0NP8bGxv/
				Jycn/y0tLf8oKCj/AAAA/wQEBP8qKir/ISEh/xwcHP8hISH/Hh4e/ysrK/8EBAT/AAAA/wAqAP8AGQD/
				AC0A/wAJAP8PDw//CQkJ/xISEv8QEBD/BAQE/wAAAP8HBwf/EhIS/xMTE/8JCQn/CgoK/wwMDP8QEBD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8IIiv/GCUp/xoaGv8WMDn/
				GzU//ysrK/82Njb/OTk5/zQ0NP8hISH/JCQk/yoqKv8oKCj/AAAA/wYGBv8kJCT/Jycn/yIiIv8kJCT/
				IiIi/zExMf8GBgb/AAAA/wEkAf8BMAH/ACIA/wAKAP8TExP/JCQk/zAwMP8fHx//GBgY/wQEBP8TExP/
				MTEx/zAwMP8TExP/Jycn/xUVFf8VFRX/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8IIiv/GCUp/xoaGv8WMDn/FzE6/ygoKP82Njb/MzMz/zAwMP8eHh7/IiIi/ycnJ/8nJyf/
				AAAA/wYGBv8eHh7/MTEx/zY2Nv8rKyv/EhIS/yUlJf8NDQ3/JCQk/w8QD/8kJCT/Kysr/wQEBP8QEBD/
				IiIi/y0tLf8bGxv/GBgY/wQEBP8QEBD/NDQ0/y0tLf8TExP/Hx8f/w0NDf8QEBD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8IIiv/GCUp/xoaGv8WMDn/HTdA/zAwMP86Ojr/
				PDw8/0BAQP8uLi7/JSUl/ygoKP8rKyv/AAAA/wYGBv8tLS3/Jycn/yIiIv8kJCT/IiIi/zAwMP8GBgb/
				AAAA/wAuAP8AIgD/ADYA/wAqAP8AKgD/ACUA/wAYAP8WFhb/GRkZ/xgYGP8NDQ3/IiIi/yEhIf8rKyv/
				LS0t/xgYGP8ZGRn/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8IIiv/
				GCUp/xoaGv8WMDn/HTdA/zAwMP86Ojr/PDw8/0BAQP8uLi7/Jycn/zAwMP8zMzP/AAAA/wYGBv8tLS3/
				Jycn/yIiIv8kJCT/IiIi/y0tLf8GBgb/AAAA/wAuAP8AIgD/ADYA/wAqAP8AKgD/ACUA/wAYAP8WFhb/
				GRkZ/xgYGP8NDQ3/IiIi/yEhIf8rKyv/LS0t/xgYGP8ZGRn/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8IIiv/GCUp/xoaGv8WMDn/FS85/yQkJP82Njb/MzMz/zo6Ov8tLS3/
				EBAQ/yIiIv8rKyv/AAAA/wQEBP8iIiL/ISEh/xkZGf8bGxv/GRkZ/ygoKP8EBAT/AAAA/wAlAP8AGwD/
				ABgA/w8PD/8eHh7/AAAA/x4eHv8VFRX/KCgo/ycnJ/8lJSX/HBwc/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8IIiv/GCUp/xoaGv8WMDn/
				GzU//y0tLf8wMDD/NjY2/zQ0NP8oKCj/JCQk/ycnJ/8rKyv/AAAA/wYGBv8qKir/JCQk/x8fH/8hISH/
				Hx8f/ysrK/8GBgb/AAAA/wAqAP8AIQD/ABUA/xUVFf8kJCT/CgoK/yIiIv8YGBj/Kysr/yoqKv8cHBz/
				Hh4e/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8IIiv/GCUp/xoaGv8WMDn/GzU//ysrK/80NDT/NjY2/zk5Of8kJCT/Hx8f/y4uLv8rKyv/
				AAAA/wQEBP8iIiL/IiIi/x8fH/8hISH/Hx8f/ysrK/8EBAT/AAAA/wAqAP8AHwD/ABYA/xUVFf8oKCj/
				BAQE/yIiIv8ZGRn/Kioq/ycnJ/8cHBz/Gxsb/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8IIiv/GCUp/xoaGv8WMDn/HTdA/ysrK/8xMTH/
				MTEx/zk5Of8qKir/Gxsb/yQkJP8qKir/AAAA/wQEBP8qKir/ISEh/xwcHP8hISH/Hh4e/ysrK/8EBAT/
				AAAA/wAqAP8AHAD/ABgA/xUVFf8nJyf/BwcH/yEhIf8VFRX/JSUl/yUlJf8iIiL/ISEh/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8IIiv/
				FC44/xYjJ/8TNkP/GzU//ysrK/86Ojr/Nzc3/zw8PP8nJyf/JCQk/yoqKv8zMzP/AAAA/wYGBv8rKyv/
				Jycn/yIiIv8iIiL/IiIi/zAwMP8GBgb/AAAA/wArAP8AIgD/ACEA/xATEP8bHhv/CQkJ/yIiIv8fHx//
				NDQ0/zQ0NP8nJyf/Gxsb/wAAAP8AAAD/AAAA/wAAAP8DAwP/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8IIiv/ABAW/wAAAP8GIy7/HTdA/y4uLv83Nzf/NDQ0/zY2Nv8zMzP/
				JSUl/zAwMP8xMTH/AAAA/wYGBv8tLS3/Jycn/yIiIv8kJCT/ISEh/zAwMP8GBgb/AAAA/wAtAP8AIgD/
				ADYA/wAqAP8AKgD/ACUA/wAYAP8WFhb/GBgY/zAwMP8NDQ3/IiIi/yEhIf8rKyv/LS0t/xkZGf8cHBz/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8IIiv/ACo5/wAaI/8GNUb/
				HTdA/zAwMP85OTn/NDQ0/z8/P/8wMDD/GRkZ/xwcHP8qKir/AAAA/wYGBv8tLS3/Jycn/yIiIv8kJCT/
				IiIi/y0tLf8GBgb/AAAA/wAzAP8AHgD/ACgA/wAKAP8ALgD/ACUA/wA3AP8AJwD/AC0A/wAlAP8AGAD/
				FhYW/x4eHv8rKyv/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8IIiv/ASs8/wE+Vv8GJTH/ByIs/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wYGBv8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8HIiz/ABEY/wElNP8GICr/DDRD/wciLP8IKTb/
				CC07/wgiK/8IIiv/CCIr/wgiK/8IIiv/CCIr/wgiK/8IIiv/CCIr/wgiK/8JKzf/Eiw1/xIsNf8SLDX/
				Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/
				Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/
				Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/
				Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/
				Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/Eiw1/xIsNf8RKzX/CCk2/wciLP8NKjX/
				EiMp/xIfI/8TLDb/BiAp/wFHYv8AFyD/ABoj/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8EERX/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/
				Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/
				Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/
				Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/
				Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/
				Ghoa/xoaGv8aGhr/ABcg/wFHYv8IFRn/Ghoa/xoaGv8YJSn/BiEr/wEqOv8AGCH/ABok/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8EEhf/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/
				Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/
				Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/
				Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/
				Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/
				Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/ABgh/wEqOv8IFhv/Ghoa/xoaGv8YJSn/
				IEpC/ypHKf8qSy7/K0wv/ytHKP8rRyj/K0co/ytHKP8rRyj/K0co/ytHKP8rRyj/Kkco/ypHKP8rTC7/
				Lkss/y5LLP8uSyz/Lkss/y5LLP8uSyz/Lkss/y5LLP8uSyz/Lkss/y5LLP8uSyz/Lkss/y5LLP8uSyz/
				Lkss/y5LLP8uSyz/Lkws/y5MLP8uTCz/Lkws/y5MLP8uTCz/Lkws/y5MLP8uTCz/Lkws/y5MLP8uTCz/
				Lkws/y5MLP8uTCz/Lkws/y1MLP8tTCz/LUws/y1MLP8tTCz/LUws/y5MLP8uTCz/Lkws/y5MLP8uTCz/
				Lkss/y5LLP8uSyz/Lkss/y5LLP8uSyz/Lkss/y5LLP8uSyz/Lkss/y5LLP8vSyz/L0ss/y9LLP8vSyz/
				L0ss/y9LLP8vSyz/L0ss/y9LLP8vSyz/L0ss/y9LLP8vSyz/JUJL/yhERP8nQ0T/J0NH/y5NMP9KSjX/
				VEk7/xo/Ov8GKzn/EjxN/wojLf8NOEj/PVo7/1tgHv9cYSP/XWMn/1tgH/9bYCD/ZGoj/3yCQ/9/hUf/
				bXMs/3F3Mv99g0T/f4VH/4mOVf+IjVP/en8+/21zLP9sciv/eH07/3B2L/9QVwH/UFcB/1BXAf9QVwH/
				UFcB/1BXAf9QVwD/UFcA/1BXAP9QVwD/UFcB/1BXAf9QVwH/UFcB/1BXAf9QVwH/UFcB/1BXAf9QVwH/
				UFcB/1BXAP9QVwD/UFcA/1BXAP9QVwD/UFcA/1BXAP9QVwD/UFcA/1BXAP9QVwD/UFcA/1BXAP9QVwD/
				UFcA/1BXAP9QVwH/UFcB/1BXAf9QVwH/UFcB/1BXAP9QVwD/UFcA/1BXAP9QVwD/UFcA/09XAP9PVwD/
				UFcA/1BXAP9QVwH/UFcB/1BXAf9QVwH/UFcB/1BXAf9QVwH/UFcB/1BXAf9QVwH/UFcB/1BXAf9QVwH/
				OT9G/z5ENv86Uz7/MGRz/0F4Wf+QTBf/ukUl/yo0FP8FFh7/JIa1/wckMf8FOk7/KlZH/ztcL/87Wiv/
				O1oq/zxbKv89XC7/PFsq/zxbKv88Wyr/PFsq/zxbKv88Wyr/PFsq/zxbKv88Wyr/PFsq/zxbKv88Wyr/
				PFsq/zxbKv88Wyr/PFsq/zxbKv88Wyr/PFsq/zxbKv88Wyr/PFwq/zxcKv88XCr/PFwp/zxcKf88XCn/
				PFwq/zxcKv88XCr/PFwq/zxcKf88XCn/PFwp/zxcKf88XCn/PFwp/zxcKf88XCn/PFwp/zxcKf88XCr/
				PVsq/z1bKv89Wyr/PVsq/z1bKv89Wyr/PVsq/z1bKv89Wyr/PVsq/z1bKv89Wyr/PVsq/z1bKv89Wyr/
				PVsq/z1bKv89Wyr/PVsq/z1bKv89Wyr/PFsq/zxbKv88Wyr/PFsq/zxbKv88Wyr/PFsq/zxbKv88Wyr/
				PFsq/zxbKv88Wyr/PFsq/zxbKv86WSz/J0Rk/ydFZP8xZEX/L2ZM/zlfNf9CWSz/UFUx/yFLPf8GLDv/
				CUhi/wU6Tv8JUGv/
			</Image>
		</BitMap>
		<Widget index="0" type="434f4e53" display_nr="3" y="3" anz_rows="5" anz_cols="16">
			<Data>
				<Data>13</Data>
				<Data>7</Data>
				<Data>0</Data>
				<Data>0</Data>
			</Data>
		</Widget>
		<Widget index="1" type="44425547" display_nr="3" has_focus="true" has_scrollfocus="true" x="5" anz_rows="3" anz_cols="11">
			<Data>
				<Data>0</Data>
				<Data>0</Data>
				<Data>0</Data>
				<Data>0</Data>
			</Data>
		</Widget>
		<Widget index="2" type="53544154" display_nr="3" has_focus="true" has_scrollfocus="true" anz_rows="3" anz_cols="5" scroll_offset="6" scroll_index="6">
			<Data>
				<Data>4</Data>
				<Data>0</Data>
				<Data>0</Data>
				<Data>0</Data>
			</Data>
		</Widget>
	</View>
</MA>]]
		
		-- File configuration
		local dir = {}
		dir.main = gma.show.getvar('PATH')..'/'
		dir.importexport = dir.main..'importexport/'
		
		fn = {}
		fn.temp = [[_templ_view.xml]]
		
		file = {}
		file.temp = dir.importexport..fn.temp
		
		-- Write temp file
		local writefile = io.open(file.temp, 'w')
		writefile:write(cmd_script)
		writefile:close()
	
		-- Import and call view
		local view_slot = advanceSpace('View', 1, 1)
		gma.cmd('SelectDrive 1')
		gma.cmd('Import "'..fn.temp..'" View '..view_slot)
		gma.cmd('View '..view_slot..' /screen='..screen)
		
		-- Cleanup
		os.remove(file.temp)
		gma.cmd('Delete View '..view_slot)
	end
} --end of gui table


local dmx = {

	--split a string to two table values
	split_addr = function(str, numindex)
	  if type(str) == 'number' then str = tostring(str) end
	  local index = str:find(':')
	  if not index then
		index = str:find('[.]')
	  end
	  local t = {}

	  if index then
		if numindex then
			t[1] = tonumber(str:sub(1, index-1))
			t[2] = tonumber(str:sub(index+1, #str))
			return t
		else
			t.univ	= tonumber(str:sub(1, index-1))
			t.ch	= tonumber(str:sub(index+1, #str))
		end
	  else
		  error('invalid address provided; no separator found')
	  end
	  
	  return t
	end,


	merge_addr = function(list, seperator)
		seperator = seperator or '.'
		
		univ = list.univ or list[1]
		ch = list.ch or list[2]
		univ, ch = tostring(univ), tostring(ch)
		
		--make ch 3 digits long
		while #ch < 3 do
			ch = '0'..ch
		end
		
		--join strings
		local t = univ..seperator..ch
		return t
	end,

	univ = {},

	univ_add = function(self, univ_num)
		if not csv.objToArray then error('Required function not found: csv.objToArray(str:obj)') end
		if not gui.superBar then error ('Required module not found: gui.superBar') end
		
		--import new universe to CSV with progress bar displayed
		pBar_add = gui.superBar:new('Importing Universe '..univ_num)
		pBar_add:setrange(0, 512)
		self.univ[univ_num] = csv.objToArray('DmxUniverse '..univ_num..'.*', pBar_add)	--import new universe
		pBar_add:stop()
		
		--[[
		for i = 1, 512 do
			self.univ[univ_num][i].addr = self.split_addr(self.univ[univ_num][i].addr, true)
		end
		--]]
	end,
	
	check_space = function(self, addr_start, spaces)
		-- if address provided in string format, convert to table object
		if spaces > 512 then spaces = 512 end	--idiot-protection
		if type(addr_start) == 'string' then addr_start = self.split_addr(addr_start) end	
		local addr_current = {univ = addr_start.univ or addr_start[1], ch = addr_start.ch or addr_start[2]}
		
		
		local confirmed = false
		local t = {ct = 0}
		
		while not confirmed do
			-- if the universe to be checked is not currently in memory, import it
			if not self.univ[addr_current.univ] then
				self:univ_add(addr_current.univ)
				--self.univ[addr_current.univ] = self:univ_add(addr_current.univ)
			end
			

			-- scan for empty channels
			for i = addr_current.ch, 512 do
				if #tostring(self.univ[addr_current.univ][i].fixture) == 0 then					--if the line has no fixture assigned
					t.ct = t.ct + 1
					if not t.addr then
						t.addr = {univ = addr_current.univ, ch = addr_current.ch}
					end
					if t.ct == spaces then
						confirmed = true
						break
					end
					addr_current.ch = i + 1
				else																			--if the line DOES have a fixture assigned
					t.ct = 0
					t.addr = nil
					addr_current.ch = i + 1
				end
			end
			
			-- if not found in that universe, reset parameters and start again at next universe number
			if not confirmed then 
				addr_current = {univ = addr_current.univ + 1, ch = 1}
				t.ct = 0
				t.addr = nil
			end		--if end of universe is reached with no match for spaces
		end

		return t.addr
	end


}


local class = {

superTable = {
	append = function(self, v)
		self[#self + 1] = v
	end,

	new = function(self, o)
		o = o or {}
		setmetatable(o, self)
		self.__index = self
		return o
	end,

	shuffle = function(list)
		local iterations = #list
		for x = 1, 4 do								--repeat process, else last number ends up the same
			for i = iterations, 1, -1 do
				j = math.random(i)
				list[i], list[j] = list[j], list[i]
			end
		end
	end
	}
	
}



---- FILE INFORMATION ----
local dir = {}
dir.main = gma.show.getvar('PATH')..'/'
dir.profiles = dir.main..'library/'
dir.importexport = dir.main..'importexport/'

local fn = {}
fn.dummy = [[@dummy_switch@.xml]]
fn.cmdview = [[_templ_cmdview.xml]]

local file = {}
file.dummy = dir.profiles..fn.dummy


-- XML scripts
local scripts = {profile_dummy = [[
<?xml version="1.0" encoding="utf-8"?>
<MA xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns="http://schemas.malighting.de/grandma2/xml/MA" xsi:schemaLocation="http://schemas.malighting.de/grandma2/xml/MA http://schemas.malighting.de/grandma2/xml/3.2.2/MA.xsd" major_vers="3" minor_vers="2" stream_vers="2">
	<Info datetime="2017-06-17T21:44:26" showfile="temp key install - pre" />
	<FixtureType index="14" name="Dummy Inverse Switch">
		<short_name>DmInvSw</short_name>
		<manufacturer>GiaffoDesigns</manufacturer>
		<short_manufacturer>GD</short_manufacturer>
		<Profiles>
			<DMX_Profile index="0" name="Switch 50%" display_spec_index="0">
				<DMX_Profile_Point index="0" mode="step_end" />
				<DMX_Profile_Point index="1" x="0.50196081" y="1" mode="step_end" />
			</DMX_Profile>
		</Profiles>
		<Modules index="0">
			<Module index="0" class="None" beamtype="Wash" beam_angle="35" beam_intensity="10000">
				<ChannelType index="0" attribute="DIM" feature="DIMMER" preset="DIMMER" coarse="1" default="0.001" highlight_value="100" snap="true" invert="true" fade_path="0">
					<ChannelFunction index="0" from="0" to="100" min_dmx_24="0" max_dmx_24="16777215" physfrom="0" physto="1" subattribute="DIM" subattribute_user_name="Dim" attribute="DIM" attribute_user_name="Dim" feature="DIMMER" feature_user_name="Dimmer" preset="DIMMER" preset_user_name="Dimmer" />
				</ChannelType>
			</Module>
		</Modules>
		<Instances index="1">
			<Instance index="0" module_index="0" />
		</Instances>
		<Wheels index="2" />
		<VirtualFunctionBlocks index="3" />
		<FixtureMacroCollect index="5" />
		<Body />
		<AutoPresets index="4" />
		<RdmNotifications index="6">
			<RdmNotification index="0" Type="Absent" Subcategory="RDM Warning" />
		</RdmNotifications>
	</FixtureType>
</MA>]]
}




-----------------------------------------------------
-------------------- MAIN SCRIPT --------------------
-----------------------------------------------------

local install_log
local install_complete

local function install_key()
	install_log = class.superTable:new()
	install_complete = false

	-------------------------
	---- SORT USERCONFIG ----
	-------------------------
	
	---- Userconfig - Temp Page ----
	local page_temp = 0
	
	-- if set to be asked during installation
	if page_temp == 0 then
		local input_msg = '[Enter Page Number]'
		local t
		repeat
			t = gma.textinput('Destination Temp Page?', input_msg)
		until (t ~= input_msg)
		
		-- if invalid input
		if (not t) or (not tonumber(t)) then 
			gma.gui.msgbox('Cancelled', 'Installation Cancelled.')
			goto EOF
		end
		
		page_temp = tonumber(t)
	end
	
	-- check if previous installations exist by looking for uninstall scripts
	local firstinstall = true
	if tempkey then
		-- break installation if attempting to install a duplicate page
		if tempkey.installed then
			for _, v in ipairs(tempkey.installed) do
				if page_temp == v then
					gui.msgbox('Installation Cancelled', 'Attempt to install duplicate temp page.\nInstallation cancelled.', 'Attempt at duplicate temp page')
					goto EOF
				end
			end
		end
	end
	
	-- Make sure page exists
	gma.cmd('Store Page '..page_temp)
	gma.cmd('Store Page '..userconfig.executor.page)


	-------------------------------------------
	---- Userconfig - plugin storage space ----
	-------------------------------------------

	-- plugin names
	local plugin_names =	{uninstall = 'Uninstall Temp '..page_temp..' Key',
							temp_on		= 'Temp-Page '..page_temp..' On',
							temp_off	= 'Temp-Page '..page_temp..' Off'}
							
							
	-- plugin configuration (names, storage slots)
	local plugins = {}
	plugins.start = advanceSpace('Plugin', userconfig.plugin_start, 3)
	plugins.uninstall	= {num = plugins.start, name = plugin_names.uninstall}
	plugins.temp_on		= {num = plugins.start + 1, name = plugin_names.temp_on}
	plugins.temp_off	= {num = plugins.start + 2, name = plugin_names.temp_off}
	
	
	
	---- CHANGE DISPLAY TO CMD/SYS VIEW ----
	gui.cmdview()	--pull up command line and system monitor on screen 2
	
	
	
	--------------------------------
	---- FIXTURETYPE OPERATIONS ----
	--------------------------------
	gma.cmd('CD /')
	gma.cmd('CD "EditSetup"')
	gma.cmd('CD "FixtureTypes"')
	local fTypes = csv.objToArray()
	
	
	local fType_num = #fTypes + 1		--number associated with FixtureType we will use later
	local fType_import = true
	for i, v in ipairs(fTypes) do
	  if v.longname == 'Dummy Inverse Switch' and v.shortname == 'DmInvSw' then		--if fixture profile already exists in showfile, will be used instead of duplicated
		fType_num = i
		fType_import = false
		install_log:append('Fixture type found already in showfile.')
		break
	  end
	end
	
	-- add dummy fixture type to showfile if not currently present
	if fType_import then
		-- write XML script to file
		local writefile = io.open(file.dummy, 'w')
		writefile:write(scripts.profile_dummy)
		writefile:close()
		
		-- import fixture profile from generated script
		gma.cmd('Import \"'..fn.dummy..'\"')
		
		-- delete temp file
		os.remove(file.dummy)
		
		install_log:append('Dummy fixture type installed.')
	end
	
	--------------------------
	---- LAYER OPERATIONS ----
	--------------------------
	gma.cmd('CD /')
	gma.cmd('CD "EditSetup"')
	gma.cmd('CD "Layers"')
	local layers = csv.objToArray()
	
	local fixture_ct = 0
	local layer_num = #layers + 1
	local layer_create = true
	
	
	for i, v in ipairs(layers) do
		if v.name:lower():find('dummy') == 1 then	-- if a layer starts with the word "dummy" 
			layer_num = i
			layer_create = false
			install_log:append('Existing "dummy" layer found. Fixture will be patched to layer "'..v.name..'".')
			
			--get fixture count information for layer
			gma.cmd('CD '..tostring(layer_num))
			fixture_ct = #csv.objToArray()
			
			break			
		end
	end
	
	--if no existing dummy layer, create one, go into new directory
	if layer_create then
		local layer_name = 'Dummy Fixtures'
		gma.cmd('Store '..tostring(layer_num))	--store new layer
		gma.cmd('Label '..tostring(layer_num)..' "'..layer_name..'"')
		gma.cmd('CD '..tostring(layer_num))
		
		install_log:append('Fixture will be patched to new layer "'..layer_name..'".')
	end
	
	---- DMX REMOTES ----
	local dmx_remotes = csv.objToArray('Remote 3.*')
	for i, v in ipairs(dmx_remotes) do
		if v.dmx ~= '-' then							-- if remote has address assigned, obtain address
			dmx_remotes[i].dmx = dmx.split_addr(v.dmx)
		else											-- else set values to 0
			dmx_remotes[i].dmx = {univ = 0, ch = 0}
		end
	end
	
	---- DMX PATCH CHECK ----
	local addr_current = {univ = userconfig.dmx_addr.univ, ch = userconfig.dmx_addr.ch}
	
	local clr_rem = false		-- trigger if empty patch spaces is clear in console patch
	local spaces_ct = 1			-- number of dmx channels needed; making a variable for potential future added features
	local addr_patch
	while not clr_rem do
		addr_patch = dmx:check_space(addr_current, spaces_ct)
		
		clr_rem = true
		for i, v in ipairs(dmx_remotes) do	
			--if the space found is in the same universe as and has overlapping channels with existing DMX remotes
			if (addr_patch.univ == dmx_remotes[i].dmx.univ) and (addr_patch.ch <= dmx_remotes[i].dmx.ch) and ((addr_patch.ch + spaces_ct - 1) >= dmx_remotes[i].dmx.ch) then
				clr_rem = false														--disable clearance trigger
				addr_current = {univ = v.dmx.univ, ch = (v.dmx.ch + 1)}				--reset starting search address one higher than the conflicting DMX-remote value
				break																--break loop
			end
		end
		
	end
	
	
	
	
	
	
	--create new fixture
	local fix_id = advanceSpace('Fixture', userconfig.fixture_num, 1) --find first available fixture number, starting with userconfig-provided number
	local fixture_pos = fixture_ct + 1
	gma.cmd('Store '..tostring(fixture_pos))						--create new fixture
	gma.cmd('Assign '..tostring(fixture_pos)..' /fixid = '..fix_id)	--assign fixture number
	gma.cmd('Assign FixtureType '..fType_num..' '..fixture_pos)		--assign fixture profile
	gma.cmd('Assign '..tostring(fixture_pos)..' /patch='..dmx.merge_addr(addr_patch))	--patch fixture
	gma.cmd('Assign '..tostring(fixture_pos)..' /reacttomaster=off')	--disable react-to-master
	gma.cmd('CD /')		--return to root directory
	
	install_log:append('Clearance found in Patch and DMX Remotes. Dummy fixture patched at '..dmx.merge_addr(addr_patch)..' with Fixture ID '..fix_id..'.')
	
	---- CREATE DMX REMOTES ----
	local command = {
		on	= [[Plugin ]]..plugins.temp_on.num,
		off	= [[Plugin ]]..plugins.temp_off.num
	}
	
	local remote = {
		off = {
			addr = dmx.merge_addr{univ = addr_patch.univ, ch = addr_patch.ch},
			pos = #dmx_remotes + 1,
			name = 'Temp '..page_temp..' Off',
			cmd = command.off
		}
	}
	
	local remote_id_str = 'Remote 3.'..remote.off.pos
	gma.cmd('Store '..remote_id_str)		--store new remotes
	for k, v in pairs(remote) do
		gma.cmd('Assign Remote 3.'..v.pos..' /type=CMD')				-- assign as command-type
		gma.cmd('Assign Remote 3.'..v.pos..' /name="'..v.name..'"')		-- assign name
		gma.cmd('Assign Remote 3.'..v.pos..' /dmx='..v.addr)			-- assign DMX address
		gma.cmd('Assign Remote 3.'..v.pos..' /cmd="'..v.cmd..'"')		-- assign command
	end
	
	install_log:append('DMX Remotes created.')
	
	---- SEQUENCE INFORMATION ----
	local seq_start = advanceSpace('Sequence', userconfig.seq_num, 1)
	
	
	---- EXECUTOR INFORMATION ----
	local exec_start = advanceExec(userconfig.executor.page, userconfig.executor.exec, 1)
	
	
	---- CREATE BUTTON ----
	-- Store to sequence
	gma.cmd('BlindEdit On')														-- blind edit
	gma.cmd('ClearAll')															-- clear blind programmer
	gma.cmd('Fixture '..fix_id)													-- select fixture
	gma.cmd('Full')																-- full
	gma.cmd('Full')																-- highlight
	gma.cmd('Store Sequence '..seq_start..' /o /so=prog /ka=false /use=active')	-- store to sequence with proper options enabled
	gma.cmd('Label Sequence '..seq_start..' Cue 1 "Page '..page_temp..'"')		-- label cue
	gma.cmd('Label Sequence '..seq_start..' "Pg '..page_temp..'"')				-- label sequence
	gma.cmd('ClearAll')															-- clear blind programmer
	gma.cmd('BlindEdit Off')													-- exit blind programmer
	
	gma.cmd('Assign Sequence '..seq_start..' Cue 1 /cmd="'..command.on..'"')	-- set command in cue
	gma.cmd('Assign Sequence '..seq_start..' Cue 1 /fade=0')					-- remove cue fade time
	gma.cmd('Lock Sequence '..seq_start)										-- lock sequence from accidental editing
	
	install_log:append('Trigger saved to Sequence '..seq_start)
	
	-- Assign to Executor, set properties
	gma.cmd('Assign Sequence '..seq_start..' Executor '..exec_start.str)
	
	local exec_props = exec_props_string(' Width=1, SwopProtect=on, KillProtect=on, AutoStart=on, AutoStop=on, AutoFix=on, OoO=off, SoftLTP=on, MasterGo=Off, Restart=First, Wrap=on, Crossfade=off, AutoStomp=on, Chaser=off, Priority=Normal, Prepos=off, EffectSpeed=on, MIBNever=off, MIBAlways=off, TriggerIsGo=off, CmdDisable=off, IgnoreExecTime=on, Speed=Normal, SpeedMaster=Speed_Individual, RateMaster=Rate_Individual, Filter=None, OffTime=Default, BreakingGo=off, PlaybackMaster=Pb None')
	gma.cmd('Assign Executor '..exec_start.str..exec_props)
	
	gma.cmd('Fix Executor '..exec_start.str)
	gma.cmd('Assign Flash Execbutton1 '..exec_start.str)
	
	install_log:append('Trigger key stored at Executor '..exec_start.str)
	
	
	
	---- CREATE PLUGINS ----
	
	-- start of triggers-creation process. Will not happen if detected to exist in the system already.
	if firstinstall then
		-- "Temp On" Plugin
		local pluginScript = [===[
--[[
Temp-Page ON
Created by Jason Giaffo
Contact: http://GiaffoDesigns.com/contact/
Last updated June 17, 2017
--]]

local page_temp = ]===]..page_temp..[===[

local function get_self_id(...)
	--CANNOT BE USED INSIDE ANOTHER FUNCTION, INCLUDING THE MAIN RETURN FUNCTION. MUST EXECUTE ON LOAD
	local unpack = table.unpack

	local self = select(2, ...)
	local indices = {0, 0}
	local self_id

	repeat
		indices = {self:find('%d+', indices[2] + 1)}
	until indices[2] == #self

	self_id = tonumber(self:sub(unpack(indices)))

	return self_id
end



-- Global Access Array for TempKey functions
if not tempkey then tempkey = {} end
if not tempkey.active_execs then tempkey.active_execs = {} end
if not tempkey.script then tempkey.script = {} end
if not tempkey.queue then tempkey.queue = {} end
if not tempkey.script[page_temp] then tempkey.script[page_temp] = {} end
local self_id = get_self_id(...)
tempkey.script[page_temp].on = self_id


local function temp_go()
	-- if another tempkey plugin is running, add self to queue, terminate self
	if tempkey.activetrig then
		tempkey.queue[#tempkey.queue + 1] = self_id
		goto EOF
	end
	
	tempkey.activetrig = true
	
	-- if no other executors are active, store source pages before moving elsewhere
	if #tempkey.active_execs == 0 then
		local faderpage_current = gma.show.getvar('faderpage')			-- get current faderpage
		local buttonpage_current = gma.show.getvar('buttonpage')		-- get current buttonpage
		gma.user.setvar('page_src_fd', faderpage_current)
		gma.user.setvar('page_src_btn', buttonpage_current)
	end
	
	-- go to temp page
	gma.cmd('Page '..page_temp)
	
	-- if page is already active ("off" did not trigger fast enough), remove duplicate entry/entries
	repeat
		local clean = true
		for i, v in ipairs(tempkey.active_execs) do
			if v == page_temp then 
				table.remove(tempkey.active_execs, i)
				clean = false
			end
		end
	until clean
	
	-- add entry for current active executor to end of list
	tempkey.active_execs[#tempkey.active_execs + 1] = page_temp
	
	
	---- Global Trigger Functions/Cleanup ----
	tempkey.activetrig = nil													-- allow other trigger plugins to run again
	if tempkey.queue[1] == self_id then table.remove(tempkey.queue, 1) end		-- remove self from queue if applicable
	if #tempkey.queue > 0 then gma.cmd('Plugin '..tempkey.queue[1]) end			-- run queue if any plugins loaded
		
	::EOF::
end

return temp_go
]===]

		createPlugin(plugins.temp_on.num, plugins.temp_on.name, pluginScript)
		gma.cmd('Lock Plugin '..plugins.temp_on.num)
		install_log:append('"Temp-Page ON" plugin created. Stored at Plugin #'..plugins.temp_on.num)
	
	
		-- "Temp Off" Plugin
		local pluginScript = [===[
--[[
Temp-Page OFF
Created by Jason Giaffo
Contact: http://GiaffoDesigns.com/contact/
Last updated June 17, 2017
--]]

local page_temp = ]===]..page_temp..[===[




local function get_self_id(...)
	--CANNOT BE USED INSIDE ANOTHER FUNCTION, INCLUDING THE MAIN RETURN FUNCTION. MUST EXECUTE ON LOAD
	local unpack = table.unpack

	local self = select(2, ...)
	local indices = {0, 0}
	local self_id

	repeat
		indices = {self:find('%d+', indices[2] + 1)}
	until indices[2] == #self

	self_id = tonumber(self:sub(unpack(indices)))

	return self_id
end



-- Global Access Array for TempKey functions
if not tempkey then tempkey = {} end
if not tempkey.script then tempkey.script = {} end
if not tempkey.script[page_temp] then tempkey.script[page_temp] = {} end
local self_id = get_self_id(...)
tempkey.script[page_temp].off = self_id


---- Main Function ----
function temp_off()
	-- if another tempkey plugin is running, add self to queue, terminate self
	if tempkey.activetrig then
		tempkey.queue[#tempkey.queue + 1] = self_id
		goto EOF
	end
	
	-- prevent other triggers from running simultaneously
	tempkey.activetrig = true
	
	repeat
		local clean = true
		for i, v in ipairs(tempkey.active_execs) do
			--if matching page is found in global list of active temp-pages, it is removed from the list and the loop is ended
			if v == page_temp then
				table.remove(tempkey.active_execs, i)
				clean = false
			end
		end
	until clean
	
	if #tempkey.active_execs == 0 then
		gma.cmd('FaderPage $page_src_fd')						--return to source pages if no other temp keys running
		gma.cmd('ButtonPage $page_src_btn')
		
		-- do not reset variables until page has been fully reset
		while gma.show.getvar('buttonpage') ~= gma.user.getvar('page_src_btn') do	
			gma.sleep(0.005)
		end
		
		-- wipe variables for "source" pages
		gma.user.setvar('page_src_fd', ' ')
		gma.user.setvar('page_src.btn', ' ')
	else
		gma.cmd('Page '..tempkey.active_execs[#tempkey.active_execs])	-- else swap to most recently triggered temp key
		
		-- do not move forward until page has been fully reset
		while tonumber(gma.show.getvar('buttonpage')) ~= tonumber(tempkey.active_execs[#tempkey.active_execs]) do
			gma.sleep(0.005)
		end
	end
	
	---- Global Trigger Functions/Cleanup ----
	tempkey.activetrig = nil													-- allow other trigger plugins to run again
	if tempkey.queue[1] == self_id then table.remove(tempkey.queue, 1) end		-- remove self from queue if applicable
	if #tempkey.queue > 0 then gma.cmd('Plugin '..tempkey.queue[1]) end			-- run queue if any plugins loaded
		
	::EOF::
end

return temp_off

]===]

		createPlugin(plugins.temp_off.num, plugins.temp_off.name, pluginScript)
		gma.cmd('Lock Plugin '..plugins.temp_off.num)
		install_log:append('"Temp-Page OFF" plugin created. Stored at Plugin #'..plugins.temp_off.num)
	
	-- end of triggers-creation process
	else
		install_log:append('Temp-Page ON and OFF plugins already exist at Plugin '..plugins.temp_on.num..' and '..plugins.temp_off.num)
	end
	
	local pluginScript = [===[
--[[
Temp-Page - UNINSTALL
Created by Jason Giaffo
Contact: http://GiaffoDesigns.com/contact/
Last updated June 15, 2017
--]]


local config = {}

config.fix_id	= ]===]..fix_id..[===[	--
config.seq_num	= ]===]..seq_start..[===[	--
config.remote = {
	off	= ']===]..remote.off.addr..[===['		--
}
config.page = ]===]..page_temp..[===[	--
config.trigplugin = {
	on	= ']===]..plugins.temp_on.num..[===[',	--
	off	= ']===]..plugins.temp_off.num..[===['
}

---- Add to global "registry" on Lua engine load ----
if not tempkey then tempkey = {} end		--
if not tempkey.installed then tempkey.installed = {} end
tempkey.installed[#tempkey.installed + 1] = config.page


local csv = {
	objToArray = function(obj, pBar)

	-------- NESTED FUNCTIONS --------

		---- converts a CSV line to a number-indexed array ----
		local function CSVlineToArray(str)
		  local t = {}
		  local indices = {1, 1}
		  local remaining = true
		  local restart = 1
		  while remaining == true do
			local pt = {str:find(',', restart)}		--find next comma at or after restart point
			if pt[1] then
			  t[#t+1] = str:sub(restart, pt[1]-1)	--extract string from restart point to one position before comma
			  restart = pt[2] + 1 					--reset restart point for one position after most recent found comma
			else
			  remaining = false
			  t[#t+1] = str:sub(restart, str[-1])
			end
		  end

		  return t
		end


		---- Converts a CSV line to an array using the indices of an input header table ----
		local function CSVsubheaderToArray(str, header)
		  local t = {}
		  local indices = {1, 1}
		  local remaining = true
		  local restart = 1
		  local index = 1
		  while remaining == true do
			local pt = {str:find(',', restart)}		--find next comma at or after restart point
			if pt[1] then
			  t[header[index]] = str:sub(restart, pt[1]-1)	--extract string from restart point to one position before comma
			  restart = pt[2] + 1 					--reset restart point for one position after most recent found comma
			  index = index + 1
			else
			  remaining = false
			  t[header[index]] = str:sub(restart, str[-1])
			end
		  end

		  return t
		end

		---- takes a CSV file, uses the first row as the headers , and indexes the contents into a table using those headers 
		local function CSVfileToArray(file, pBar)
			local header = {}
			local results = {}

			local firstLine = true
			for line in io.lines(file) do
			  if firstLine then
				header = CSVlineToArray(line)
				for k, v in pairs(header) do header[k] = v:lower() end
				firstLine = false
			  else
				results[#results+1] = CSVsubheaderToArray(line, header)
				if pBar then pBar:set(1, true) end
			  end
			end

			return results
		  
		end
		
		
		
		---- main function ----
		-- filepaths
		local dir	= {temp = gma.show.getvar('PATH')..'/reports/'}
		local fn 	= {temp = 'export_csvobject.csv'}
		local file	= {temp = dir.temp..fn.temp}
		
		-- export object
		gma.cmd('SelectDrive 1')
		if obj then	gma.cmd('List '..obj..' /f=\"'..fn.temp..'\"')
		else gma.cmd('List /f=\"'..fn.temp..'\"') end
		
		local t		--declare before if statement
		-- check if file exists; if not, no object was found for export
		local test = io.open(file.temp)
		if test then									--if file is present
			test:close()								--close test file
			
			-- convert into table
			t = CSVfileToArray(file.temp, pBar)
				
			-- delete temp file
			os.remove(file.temp)
		else
			t = {}
		end

		
		-- return result
		return t
	end,
	

	
	---- function to take a two-dimensional table and convert it to a CSV text string
	write_table = function(list)
		-- nested function to write a single line
		local function write_line(list)
			local t = list[1]
			if #list > 1 then
			  for i = 2, #list do
				t = t..','..list[i]
			  end
		    end
		  
			if t == nil then t = '' end
		  
			return t
		end
	
		-- main function
		local t = ''
		for i = 1, #list do
		  t = t..write_line(list[i])..'\n'
		end

		return t
	end
}


local class = {

superTable = {
	append = function(self, v)
		self[#self + 1] = v
	end,

	new = function(self, o)
		o = o or {}
		setmetatable(o, self)
		self.__index = self
		return o
	end,

	shuffle = function(list)
		local iterations = #list
		for x = 1, 4 do								--repeat process, else last number ends up the same
			for i = iterations, 1, -1 do
				j = math.random(i)
				list[i], list[j] = list[j], list[i]
			end
		end
	end
	}
	
}


local gui = {
-- cmdview() requires advanceSpace function
	superBar = {
		value = 0,
		
		new = function(self, name)
			o = {handle = gma.gui.progress.start(name)}

			setmetatable(o, self)
			self.__index = self
			return o
		end,
		
		set = function(self, num, add)
			if add then num = self.value + num end
			
			num = math.floor(num)
			gma.gui.progress.set(self.handle, num)
			self.value = num
		end,
		
		setrange = function(self, bottom, top)
			if not bottom or not top then return nil end
			gma.gui.progress.setrange(self.handle, bottom, top)
			self.range = {top = top, bottom = bottom}
		end,
		
		settext = function(self, text)
			gma.gui.progress.settext(self.handle, text)
		end,
		
		rename = function(self, name)
			gma.gui.progress.stop(self.handle)
			self.handle = gma.gui.progress.start(name)
			if self.range then gma.gui.progress.setrange(self.handle, self.range.bottom, self.range.top) end
		end,
		
		stop = function(self)
			gma.gui.progress.stop(self.handle)
		end,
		
		time_move = function(self, target, time, add, ignore_top)
			if add then target = self.value + target end
			local start = self.value
			local finish = math.floor(target)
			
			local sleepPeriod
			if math.abs(time) ~= time then sleepPeriod = (1 / time)											--allow for negative time values to trigger a "rate" function
			else sleepPeriod = time / (math.abs(finish - start)) end
			
			if not ignore_top and self.range and finish > self.range.top then finish = self.range.top end	--set finish point to top of progress-bar range unless specified to ignore
			
			local dir = 1
			if start > finish then dir = -1 end
			
			for i = start, finish, dir do
				self:set(i)
				gma.sleep(sleepPeriod)
			end	
		end
	}, --end of superBar object
	
	
	msgbox = function(title, message_box, message_text)
	  -- v. 1.0
	  -- function avoids using confirmation box function with version 3.1.2.5, where it doesn't exist
	  local confirm_method
	  
	  local version = gma.show.getvar('version')
	  
	  if version:find('3.1.2') == 1 then 
		confirm_method = 'textinput'
	  else 
		confirm_method = 'box' end
	  
	  if confirm_method == 'box' then
		gma.gui.msgbox(title, message_box)
	  elseif confirm_method == 'textinput' then
		gma.textinput(title, message_text)
	  end
	end,
	
	
	confirm = function(title, message_box, message_text)
	  -- v. 1.0
	  -- function avoids using confirmation box function with version 3.1.2.5, which crashes the software
	  local confirm_method
	  
	  local version = gma.show.getvar('version')
	  
	  if version:find('3.1.2') == 1 then 
		confirm_method = 'textinput'
	  else 
		confirm_method = 'box' end
	  
	  if confirm_method == 'box' then
		return gma.gui.confirm(title, message_box)
	  elseif confirm_method == 'textinput' then
		local t = gma.textinput(title, message_text)
		if t then t = true end
		return t
	  end
	end,
	
	
	print = function(str)
		gma.echo(str)
		gma.feedback(str)
	end,
	
	
	cmdview = function(screen)
		if not screen then screen = 2 end
		local cmd_script = [[
<?xml version="1.0" encoding="utf-8"?>
<MA xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns="http://schemas.malighting.de/grandma2/xml/MA" xsi:schemaLocation="http://schemas.malighting.de/grandma2/xml/MA http://schemas.malighting.de/grandma2/xml/3.2.2/MA.xsd" major_vers="3" minor_vers="2" stream_vers="2">
	<Info datetime="2017-06-16T16:29:12" showfile="temp key install - pre" />
	<View index="31" name="Sys" display_mask="8">
		<BitMap width="96" height="48">
			<Image>
				dBUY/09RYP95Cgz/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+DBwf/
				gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ACAv9+BAT/
				cBET/09RYP95Cgz/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/
				gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4MGBv+BAwP/
				gQIC/4ECAv+BAgL/gQIC/4EDA/+DBwf/gQIC/4ECAv+CBAT/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/
				gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/
				gQIC/4ECAv+BAgL/gAIC/4ACAv9/BAT/RlRk/wSryf9NKDD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/
				fwAA/38AAP9/AAD/fwAA/38AAP+yZWX/qVVV/7hycv+pVVX/s2ho/4UMDP9/AAD/fwAA/38AAP9/AAD/
				fwAA/38AAP9/AAD/fwAA/38AAP+BAgL/SEhU/wSryf9NKDD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/
				fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/
				fwAA/38AAP9/AAD/fwAA/6dRUf+vYWH/pUtL/6tZWf+uXl7/tW1t/5YuLv/Gjo7/qlVV/6pVVf+2cHD/
				qFJS/6VMTP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/
				fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP+BAgL/cQ4Q/0YnLv92Bwf/
				gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+DBwf/gwYG/4QICP+ECAj/
				hAkJ/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ACAv9+BAT/bQ0O/0YnLv92Bwf/
				gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/
				gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4MHB/+MFxf/gwYG/4MFBf+ECAj/
				hAgI/4IEBP+DBgb/gwYG/4MHB/+DBwf/hAgI/4IFBf+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/
				gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/
				gAIC/4ACAv9/BAT/EzxL/yE7RP8RKzX/GjQ+/wwlL/8QKjT/FS84/xUvOf8TLTb/BiAp/wYgKf8GICn/
				BiAp/wYgKf8GICn/BiAp/wYgKf8GICn/BiAp/wYgKf8GICn/BiAp/wYgKf8GICn/BiAp/wYgKf8JMUH/
				CR4l/wQSGP8GHCX/FDlH/yA7RP8pQ03/HDY//x85Qv8cNj//FC44/yA6RP8fOUL/Ey02/yA6RP8WMDn/
				HzlC/xkzPP8ULjj/CyUv/wslL/8gOkT/FjA5/xkzPP8ZMzz/JT9I/yI8Rf8WMDn/HTdB/yI8Rf8OKDL/
				FzE7/xkzPP8gOkT/FjA5/xo0Pv8NJzD/HTdB/xcxO/8RKzX/FzE7/w4oMv8TLTb/ECoz/x85Qv8jPUf/
				ESs1/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/
				CiQt/wokLf8KJC3/CiMt/wkkLv8OMT7/CCk2/wciLP8LMkH/FzhF/yUyN/8mMzj/KDU6/w8bIP8hLjP/
				LTk+/zE+Q/8TICX/AA0R/wANEf8ADRH/AA0R/wANEf8ADRH/AA0R/wANEf8ADRH/AA0R/wANEf8ADRH/
				AA0R/wMOEv8ADRH/AA0R/wANEf8EJjL/Bh4n/wJcgP8CKDb/GjQ+/ysrK/8zMzP/MTEx/zExMf8iIiL/
				Hx8f/yoqKv8iIiL/DQ0N/y4uLv8bGxv/Kioq/x4eHv8uLi7/BgYG/wYGBv8oKCj/IiIi/xwcHP8ZGRn/
				LS0t/ysrK/8fHx//HBwc/y0tLf8PDw//FhYW/ycnJ/8eHh7/Hh4e/x8fH/8PDw//Jycn/yoqKv8VFRX/
				GRkZ/wkJCf8MDAz/Dw8P/x8fH/8kJCT/EhIS/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8IISv/ACMx/wFObP8GIy7/
				BBEV/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/xMLA/8AAAD/AAAA/wAAAP8GExf/CSIr/wQSGP8FISz/
				FjA5/ygoKP8wMDD/LS0t/zExMf8bGxv/Gxsb/yQkJP8oKCj/Dw8P/zY2Nv8fHx//JCQk/yIiIv8iIiL/
				BgYG/wYGBv8kJCT/IiIi/xISEv8GBgb/Kioq/yQkJP8eHh7/Kysr/xgYGP8kJCT/MTEx/yUlJf8TExP/
				GRkZ/wAAAP8SEhL/Hx8f/yUlJf8nJyf/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8IIiv/ACAs/wATG/8GJzP/BBEV/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/xMLA/8AAAD/
				AAAA/wAAAP8GExf/GCUp/xoaGv8YJSn/HDY//y4uLv83Nzf/NDQ0/zc3N/8lJSX/Hh4e/y4uLv8zMzP/
				AAAA/wAAAP8xMTH/Ojo6/xYWFv8GBgb/ExMT/zk5Of8eHh7/NDQ0/zQ0NP8KCgr/HBwc/ygoKP8rKyv/
				IiIi/y4uLv8KCgr/NjY2/ycnJ/8cHBz/Hx8f/w0NDf8GBgb/FhYW/ygoKP80NDT/FhYW/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8IIiv/Fi43/xglKf8VNUH/BCMW/wATAP8AEwD/
				ABMA/wATAP8AEwD/ABMA/wATAP8AEwD/ABMA/wATAP8AEwD/ABMA/wATAP8AEwD/ABMA/wATAP8AEwD/
				ABMA/wATAP8AEwD/ABMA/xAaA/8AAAD/AAAA/wAAAP8FEhf/GCUp/xoaGv8YJSn/GjQ+/zAwMP85OTn/
				NDQ0/zc3N/8lJSX/Hh4e/y4uLv8zMzP/EBAQ/zc3N/8hISH/Li4u/x8fH/8tLS3/BgYG/wYGBv8tLS3/
				JSUl/xYWFv8hISH/OTk5/zk5Of8TExP/Jycn/zo6Ov8NDQ3/ISEh/zAwMP8tLS3/JSUl/ysrK/8KCgr/
				MDAw/yUlJf8VFRX/IiIi/wwMDP8TExP/ExMT/zQ0NP8zMzP/FhYW/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8IIiv/
				GCUp/xoaGv8WMDn/Gk8t/ysrK/8nJyf/MTEx/yQkJP8hISH/IiIi/w8PD/8bGxv/JCQk/yMjI/8WFhb/
				Dg4O/ykpKf8pKSn/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8EERX/
				GCUp/xoaGv8YJSn/GjQ+/zAwMP85OTn/NDQ0/zY2Nv8zMzP/Hx8f/zY2Nv8xMTH/AAAA/wAAAP8xMTH/
				Ojo6/xYWFv8GBgb/DQ0N/ycnJ/8rKyv/Kioq/x8fH/8JCQn/EBAQ/zo6Ov85OTn/GBgY/zQ0NP8fHx//
				ExMT/xwcHP8TExP/PT09/xMTE/8qKir/Ojo6/ycnJ/8TExP/BgYG/xsbG/8nJyf/Kioq/zQ0NP8kJCT/
				Jycn/yIiIv8VFRX/BgYG/zMzM/8SEhL/JCQk/xsbG/8uLi7/LS0t/ycnJ/8lJSX/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8IIiv/GCUp/xoaGv8WMDn/EUI4/xQhJv8YJSr/FSIn/xQhJf8PHCH/
				EyAl/w4bIP8WIyj/FSIn/xMgJf8NGh//Cxgd/w4bH/8SHiP/AA0R/wANEf8ADRH/AA0R/wANEf8ADRH/
				AA0R/wANEf8ADRH/AA0R/wANEf8EICv/GCUp/xoaGv8YJSn/GTM8/y0tLf8zMzP/LS0t/zc3N/8oKCj/
				FRUV/x4eHv8rKyv/DQ0N/ysrK/8ZGRn/MTEx/x8fH/8eHh7/BAQE/w8PD/8rKyv/Jycn/ysrK/8bGxv/
				Kioq/yEhIf8wMDD/Hx8f/zExMf8fHx//LS0t/yEhIf8uLi7/ISEh/ysrK/8kJCT/Kysr/yEhIf8oKCj/
				JSUl/ycnJ/8uLi7/Kioq/y4uLv8tLS3/MDAw/x8fH/8tLS3/Hx8f/zQ0NP8cHBz/MTEx/yEhIf8wMDD/
				ISEh/y0tLf8cHBz/Jycn/xsbG/8nJyf/Gxsb/xISEv8AAAD/AAAA/wAAAP8IIiv/GCUp/xoaGv8WMDn/
				HTpF/z09Pf86Ojr/Li4u/zQ0NP8jIyP/JSUl/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8EISv/CSEq/wQRFf8FHyn/
				GTM8/y0tLf82Njb/MDAw/zo6Ov8rKyv/GBgY/xsbG/8qKir/Dw8P/y0tLf8eHh7/KCgo/yQkJP8kJCT/
				BgYG/wYGBv8TExP/ExMT/xwcHP8hISH/HBwc/xsbG/8PDw//Hx8f/x8fH/8cHBz/BwcH/wwMDP8fHx//
				HBwc/xISEv8VFRX/FhYW/xgYGP8KCgr/HBwc/x8fH/8cHBz/Hx8f/xwcHP8WFhb/IiIi/xUVFf8bGxv/
				Gxsb/xMTE/8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8IIiv/CSQu/woXG/8MMD7/BCsk/wAvEf8ASRH/ADwR/wASEf8ADRH/AA0R/wANEf8ADRH/
				AA0R/wANEf8ADRH/AA0R/wAQEf8ARhH/ADUR/wANEf8ADRH/AA0R/wANEf8ADRH/AA0R/wANEf8ADRH/
				AA0R/wANEf8GHSb/CiIr/wYTF/8HISv/GjQ+/ysrK/8tLS3/MDAw/zMzM/8rKyv/GBgY/xsbG/8rKyv/
				Dw8P/zAwMP8cHBz/JSUl/yQkJP8kJCT/BAQE/wQEBP8YGBj/JCQk/yUlJf8qKir/Jycn/ygoKP8lJSX/
				Jycn/zQ0NP8tLS3/JCQk/yEhIf80NDT/Hx8f/yUlJf8nJyf/MTEx/ycnJ/8hISH/Li4u/zExMf8uLi7/
				MTEx/y4uLv8rKyv/MzMz/yUlJf8xMTH/Hx8f/ycnJ/8NDQ3/FRUV/w0NDf8VFRX/DQ0N/xUVFf8NDQ3/
				ExMT/w0NDf8TExP/DQ0N/wkJCf8AAAD/AAAA/wAAAP8IIiv/ABsl/wANEf8GKjj/BCkV/wAkAP8AIgD/
				ABwA/wAFAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAEAP8ALgD/ACkA/wADAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8GExf/GCUp/xoaGv8YJSn/HDY//ysrK/8zMzP/
				MTEx/zY2Nv8tLS3/GBgY/xgYGP8qKir/CQkJ/ygoKP8nJyf/LS0t/ygoKP8WFhb/CQkJ/w0NDf8VFRX/
				Hh4e/xMTE/8NDQ3/GBgY/xsbG/8fHx//ExMT/x8fH/8WFhb/HBwc/xYWFv8bGxv/HBwc/xkZGf8cHBz/
				JCQk/xkZGf8cHBz/EhIS/xsbG/8VFRX/EhIS/xISEv8YGBj/FRUV/w0NDf8eHh7/EBAQ/xYWFv8NDQ3/
				Gxsb/w8PD/8oKCj/EBAQ/xUVFf8NDQ3/ExMT/w0NDf8TExP/DQ0N/wkJCf8AAAD/AAAA/wAAAP8IIiv/
				ACUz/wAYIf8GLDv/BDEV/wBDAP8AMwD/ACEA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAEAP8AUgD/ADoA/wAhAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8GExf/
				FSw1/xYjJ/8ULTf/FjA5/ycnJ/8uLi7/LS0t/zc3N/8oKCj/FRUV/yIiIv8qKir/AAAA/wAAAP8oKCj/
				Li4u/wwMDP8GBgb/DAwM/yEhIf8qKir/ISEh/xgYGP8EBAT/DQ0N/zMzM/8nJyf/FRUV/ysrK/8YGBj/
				DQ0N/xYWFv8PDw//Nzc3/w8PD/8fHx//MDAw/x8fH/8MDAz/BgYG/xkZGf8iIiL/Hx8f/xwcHP8YGBj/
				Kioq/yUlJf8uLi7/ISEh/yUlJf8iIiL/EhIS/wcHB/8tLS3/DAwM/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8IISv/ACg4/wJUdP8GJDD/BRIX/wAEAP8AAQD/AAQA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAwD/AAMA/wAEAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8GExf/Bh0m/wAUHP8CHyv/DjE//wciLP8IISv/CCIr/wgiK/8IIiv/
				CCIr/wgiK/8IIiv/CCIr/wgiK/8IIiv/CCIr/wgiK/8IIiv/CCIr/wgiK/8IIiv/CCIr/wgiK/8IIiv/
				CCIr/wgiK/8IIiv/CCIr/wgiK/8IIiv/CCIr/wgiK/8IIiv/CCIr/wgiK/8IIiv/CCIr/wgiK/8IIiv/
				CCIr/wgiK/8IIiv/CCIr/wgiK/8IIiv/CCIr/wgiK/8IIiv/CCIr/wgiK/8IIiv/CCIr/wgiK/8IIiv/
				CCIr/wgiK/8IIiv/CCIr/wgiK/8IIiv/CCIr/wgiK/8IIiv/CCEr/wciLP8NLTn/CB4m/wgVGf8MKjb/
				Cxgd/wgICP8ICAj/CAgI/wgICP8ICAj/CAgI/wgICP8ICAj/CAgI/wgICP8ICAj/CAgI/wgICP8ICAj/
				CAgI/wgICP8ICAj/CAgI/wgICP8ICAj/CAgI/wgICP8ICAj/CAgI/wgICP8LGB3/Bhkh/wFRcP8BIS7/
				CCs5/wAoOP8AJTP/ACY0/wANEf8ADRH/AA0R/wAOE/8SLTf/GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/
				GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/
				GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/
				GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/
				ACUz/wAoOP8IHib/Ghoa/xoaGv8WJiz/GCUp/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/
				Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/
				Ghoa/xoaGv8YJSn/Dh8m/wgaIf8HMkL/ByYy/wJUdP8AGCH/ABoj/wAAAP8AAAD/AAAA/wAAAP8UISX/
				Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/
				Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/
				Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/
				Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/ABgh/wJUdP8IFRn/Ghoa/xgfIf8TNEH/Fi02/xgkKf8YJSn/
				GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/
				GCUp/xglKf8YJSn/GCUp/xglKf8YJSn/GCUp/xgkKf8WLTb/GB8h/xMyPv8TPU3/CzdI/wYkMP8GLDv/
				BjFB/wYgKf8GICn/BiAp/wYhK/8SNkP/FjA5/xYwOf8WMDn/FjA5/xYwOf8WMDn/FjA5/xYwOf8WMDn/
				FjA5/xYwOf8WMDn/FjA5/xYwOf8WMDn/FjA5/xYwOf8WMDn/FjA5/xYwOf8WMDn/FjA5/xYwOf8WMDn/
				FjA5/xYwOf8WMDn/FjA5/xYwOf8WMDn/FjA5/xYwOf8WMDn/FjA5/xYwOf8WMDn/FjA5/xYwOf8WMDn/
				FjA5/xYwOf8WMDn/FjA5/xYwOf8WMDn/FjA5/xYwOf8WMDn/FjA5/xYwOf8WMDn/Biw7/wYkMP8MKjb/
				FiYs/xM0Qf8SQVP/dBUY/09RYP95Cgz/gAIC/4ECAv+BAgL/ggMD/4ECAv+BAgL/gQIC/4ECAv+BAgL/
				ggMD/4ECAv+BAgL/gQIC/4ECAv+BAgL/ggMD/4ECAv+BAgL/gQIC/4ECAv+BAgL/ggMD/4ECAv+BAgL/
				gQIC/4ECAv+BAgL/ggMD/4ECAv+BAgL/gQIC/4ECAv+BAgL/ggMD/4ECAv+BAgL/gQIC/4MGBv+BAgL/
				ggMD/4ECAv+BAgL/gQIC/4ECAv+CBAT/hQgI/4ECAv+BAgL/gwYG/4IFBf+BAgL/ggQE/4MHB/+BAgL/
				gQIC/4IEBP+BAgL/ggMD/4ECAv+BAgL/gQIC/4ECAv+BAgL/ggMD/4ECAv+BAgL/gQIC/4ECAv+BAgL/
				ggMD/4ECAv+BAgL/gQIC/4ECAv+BAgL/ggMD/4ECAv+BAgL/gQIC/4ECAv+BAgL/ggMD/4ECAv+BAgL/
				gQIC/4ECAv+BAgL/ggMD/4ECAv+AAgL/gAIC/4ACAv9/BAT/RlRk/wSryf9NKDD/fwAA/38AAP9/AAD/
				fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/
				fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/
				fwAA/38AAP9/AAD/fwAA/6ZPT/+pVFT/rFpa/6lVVf+rWFj/qFNT/6pWVv+zaWn/smdn/6tXV/+oUlL/
				n0BA/6dQUP+tXFz/rFtb/8KFhf+uXl7/rFlZ/7JlZf+ECQn/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/
				fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/
				fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP9/AAD/fwAA/38AAP+BAgL/
				cQ4Q/0YnLv92Bwf/gAIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/
				gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/
				gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4MGBv+DBgb/gwYG/4MGBv+DBwf/
				gwcH/4MHB/+DBwf/gwYG/4MGBv+DBgb/ggQE/4IFBf+DBwf/gwcH/4QJCf+DBwf/hAgI/4MHB/+CBAT/
				gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/
				gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/gQIC/4ECAv+BAgL/
				gQIC/4ECAv+AAgL/gAIC/4ACAv9/BAT/DTRD/wkkLv8JJC3/CSMt/wokLf8KJC3/CiQt/wokLf8KJC3/
				CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/
				CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/
				CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/
				CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/
				CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/CiQt/wokLf8KJC3/
				CiQt/wokLf8KJC3/CiQt/wokLf8KIy3/CSQt/wkkLv8NMT7/CCo2/wciLP8LM0H/Dykz/w0NDf8KCgr/
				DAwM/wkJCf8KCgr/BgYG/woKCv8JCQn/AAAA/wAAAP8DAwP/AAAA/wAAAP8EBAT/AQEB/wEBAf8AAAD/
				AAAA/wMDA/8DAwP/AAAA/wAAAP8DAwP/AAAA/wAAAP8DAwP/AwMD/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8IISv/
				ACMx/wFObP8GIy7/FzE6/ygoKP82Njb/NDQ0/zc3N/8oKCj/FRUV/y0tLf8rKyv/AAAA/wYGBv8eHh7/
				MTEx/zY2Nv8tLS3/EhIS/ycnJ/8NDQ3/JCQk/w8PD/8kJCT/Kysr/wQEBP8ZGRn/Li4u/zQ0NP8ZGRn/
				BAQE/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8IIiv/ACAs/wATG/8GJzP/HTdA/y4uLv85OTn/PDw8/z8/P/8wMDD/
				JSUl/zo6Ov80NDT/AAAA/wYGBv8fHx//MTEx/zY2Nv8tLS3/EBAQ/yUlJf8NDQ3/JCQk/xAQEP8lJSX/
				Kysr/wQEBP8WFhb/MTEx/zY2Nv8cHBz/Kioq/xsbG/8tLS3/Jycn/yIiIv8GBgb/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8IIiv/Fi43/xglKf8VNUH/
				HTdA/zAwMP86Ojr/PDw8/z8/P/8wMDD/KCgo/zExMf8zMzP/AAAA/wYGBv8fHx//Li4u/zMzM/8uLi7/
				EhIS/ycnJ/8NDQ3/JCQk/xAQEP8lJSX/Kysr/wQEBP8WFhb/MzMz/zc3N/8cHBz/Kysr/xsbG/8tLS3/
				Jycn/yIiIv8GBgb/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8IIiv/GCUp/xoaGv8WMDn/HTdA/zAwMP86Ojr/MDAw/z09Pf8kJCT/Jycn/zMzM/8zMzP/
				AAAA/wYGBv8tLS3/LS0t/ygoKP8kJCT/IiIi/ygoKP8GBgb/AAAA/wA2AP8AIQD/ACsA/wAKAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8IIiv/GCUp/xoaGv8WMDn/GzU//y0tLf82Njb/
				Li4u/zExMf8cHBz/IiIi/ysrK/8nJyf/AAAA/wQEBP8qKir/IiIi/x4eHv8hISH/Hx8f/yoqKv8EBAT/
				AAAA/wAxAP8AHgD/ACsA/wAJAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8IIiv/
				GCUp/xoaGv8WMDn/GzU//y0tLf80NDT/Nzc3/zc3N/8kJCT/JCQk/yUlJf8tLS3/AAAA/wYGBv8rKyv/
				Hh4e/x8fH/8hISH/Hx8f/ysrK/8GBgb/AAAA/wAuAP8AHAD/ACsA/wAJAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8IIiv/GCUp/xoaGv8WMDn/HTdA/ysrK/8xMTH/MTEx/zQ0NP8bGxv/
				Jycn/y0tLf8oKCj/AAAA/wQEBP8qKir/ISEh/xwcHP8hISH/Hh4e/ysrK/8EBAT/AAAA/wAqAP8AGQD/
				AC0A/wAJAP8PDw//CQkJ/xISEv8QEBD/BAQE/wAAAP8HBwf/EhIS/xMTE/8JCQn/CgoK/wwMDP8QEBD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8IIiv/GCUp/xoaGv8WMDn/
				GzU//ysrK/82Njb/OTk5/zQ0NP8hISH/JCQk/yoqKv8oKCj/AAAA/wYGBv8kJCT/Jycn/yIiIv8kJCT/
				IiIi/zExMf8GBgb/AAAA/wEkAf8BMAH/ACIA/wAKAP8TExP/JCQk/zAwMP8fHx//GBgY/wQEBP8TExP/
				MTEx/zAwMP8TExP/Jycn/xUVFf8VFRX/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8IIiv/GCUp/xoaGv8WMDn/FzE6/ygoKP82Njb/MzMz/zAwMP8eHh7/IiIi/ycnJ/8nJyf/
				AAAA/wYGBv8eHh7/MTEx/zY2Nv8rKyv/EhIS/yUlJf8NDQ3/JCQk/w8QD/8kJCT/Kysr/wQEBP8QEBD/
				IiIi/y0tLf8bGxv/GBgY/wQEBP8QEBD/NDQ0/y0tLf8TExP/Hx8f/w0NDf8QEBD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8IIiv/GCUp/xoaGv8WMDn/HTdA/zAwMP86Ojr/
				PDw8/0BAQP8uLi7/JSUl/ygoKP8rKyv/AAAA/wYGBv8tLS3/Jycn/yIiIv8kJCT/IiIi/zAwMP8GBgb/
				AAAA/wAuAP8AIgD/ADYA/wAqAP8AKgD/ACUA/wAYAP8WFhb/GRkZ/xgYGP8NDQ3/IiIi/yEhIf8rKyv/
				LS0t/xgYGP8ZGRn/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8IIiv/
				GCUp/xoaGv8WMDn/HTdA/zAwMP86Ojr/PDw8/0BAQP8uLi7/Jycn/zAwMP8zMzP/AAAA/wYGBv8tLS3/
				Jycn/yIiIv8kJCT/IiIi/y0tLf8GBgb/AAAA/wAuAP8AIgD/ADYA/wAqAP8AKgD/ACUA/wAYAP8WFhb/
				GRkZ/xgYGP8NDQ3/IiIi/yEhIf8rKyv/LS0t/xgYGP8ZGRn/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8IIiv/GCUp/xoaGv8WMDn/FS85/yQkJP82Njb/MzMz/zo6Ov8tLS3/
				EBAQ/yIiIv8rKyv/AAAA/wQEBP8iIiL/ISEh/xkZGf8bGxv/GRkZ/ygoKP8EBAT/AAAA/wAlAP8AGwD/
				ABgA/w8PD/8eHh7/AAAA/x4eHv8VFRX/KCgo/ycnJ/8lJSX/HBwc/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8IIiv/GCUp/xoaGv8WMDn/
				GzU//y0tLf8wMDD/NjY2/zQ0NP8oKCj/JCQk/ycnJ/8rKyv/AAAA/wYGBv8qKir/JCQk/x8fH/8hISH/
				Hx8f/ysrK/8GBgb/AAAA/wAqAP8AIQD/ABUA/xUVFf8kJCT/CgoK/yIiIv8YGBj/Kysr/yoqKv8cHBz/
				Hh4e/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8IIiv/GCUp/xoaGv8WMDn/GzU//ysrK/80NDT/NjY2/zk5Of8kJCT/Hx8f/y4uLv8rKyv/
				AAAA/wQEBP8iIiL/IiIi/x8fH/8hISH/Hx8f/ysrK/8EBAT/AAAA/wAqAP8AHwD/ABYA/xUVFf8oKCj/
				BAQE/yIiIv8ZGRn/Kioq/ycnJ/8cHBz/Gxsb/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8IIiv/GCUp/xoaGv8WMDn/HTdA/ysrK/8xMTH/
				MTEx/zk5Of8qKir/Gxsb/yQkJP8qKir/AAAA/wQEBP8qKir/ISEh/xwcHP8hISH/Hh4e/ysrK/8EBAT/
				AAAA/wAqAP8AHAD/ABgA/xUVFf8nJyf/BwcH/yEhIf8VFRX/JSUl/yUlJf8iIiL/ISEh/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8IIiv/
				FC44/xYjJ/8TNkP/GzU//ysrK/86Ojr/Nzc3/zw8PP8nJyf/JCQk/yoqKv8zMzP/AAAA/wYGBv8rKyv/
				Jycn/yIiIv8iIiL/IiIi/zAwMP8GBgb/AAAA/wArAP8AIgD/ACEA/xATEP8bHhv/CQkJ/yIiIv8fHx//
				NDQ0/zQ0NP8nJyf/Gxsb/wAAAP8AAAD/AAAA/wAAAP8DAwP/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8IIiv/ABAW/wAAAP8GIy7/HTdA/y4uLv83Nzf/NDQ0/zY2Nv8zMzP/
				JSUl/zAwMP8xMTH/AAAA/wYGBv8tLS3/Jycn/yIiIv8kJCT/ISEh/zAwMP8GBgb/AAAA/wAtAP8AIgD/
				ADYA/wAqAP8AKgD/ACUA/wAYAP8WFhb/GBgY/zAwMP8NDQ3/IiIi/yEhIf8rKyv/LS0t/xkZGf8cHBz/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8IIiv/ACo5/wAaI/8GNUb/
				HTdA/zAwMP85OTn/NDQ0/z8/P/8wMDD/GRkZ/xwcHP8qKir/AAAA/wYGBv8tLS3/Jycn/yIiIv8kJCT/
				IiIi/y0tLf8GBgb/AAAA/wAzAP8AHgD/ACgA/wAKAP8ALgD/ACUA/wA3AP8AJwD/AC0A/wAlAP8AGAD/
				FhYW/x4eHv8rKyv/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8IIiv/ASs8/wE+Vv8GJTH/ByIs/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wYGBv8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8HIiz/ABEY/wElNP8GICr/DDRD/wciLP8IKTb/
				CC07/wgiK/8IIiv/CCIr/wgiK/8IIiv/CCIr/wgiK/8IIiv/CCIr/wgiK/8JKzf/Eiw1/xIsNf8SLDX/
				Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/
				Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/
				Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/
				Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/
				Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/Eiw1/xIsNf8SLDX/Eiw1/xIsNf8RKzX/CCk2/wciLP8NKjX/
				EiMp/xIfI/8TLDb/BiAp/wFHYv8AFyD/ABoj/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/
				AAAA/wAAAP8EERX/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/
				Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/
				Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/
				Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/
				Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/
				Ghoa/xoaGv8aGhr/ABcg/wFHYv8IFRn/Ghoa/xoaGv8YJSn/BiEr/wEqOv8AGCH/ABok/wAAAP8AAAD/
				AAAA/wAAAP8AAAD/AAAA/wAAAP8AAAD/AAAA/wAAAP8EEhf/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/
				Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/
				Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/
				Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/
				Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/
				Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/Ghoa/xoaGv8aGhr/ABgh/wEqOv8IFhv/Ghoa/xoaGv8YJSn/
				IEpC/ypHKf8qSy7/K0wv/ytHKP8rRyj/K0co/ytHKP8rRyj/K0co/ytHKP8rRyj/Kkco/ypHKP8rTC7/
				Lkss/y5LLP8uSyz/Lkss/y5LLP8uSyz/Lkss/y5LLP8uSyz/Lkss/y5LLP8uSyz/Lkss/y5LLP8uSyz/
				Lkss/y5LLP8uSyz/Lkws/y5MLP8uTCz/Lkws/y5MLP8uTCz/Lkws/y5MLP8uTCz/Lkws/y5MLP8uTCz/
				Lkws/y5MLP8uTCz/Lkws/y1MLP8tTCz/LUws/y1MLP8tTCz/LUws/y5MLP8uTCz/Lkws/y5MLP8uTCz/
				Lkss/y5LLP8uSyz/Lkss/y5LLP8uSyz/Lkss/y5LLP8uSyz/Lkss/y5LLP8vSyz/L0ss/y9LLP8vSyz/
				L0ss/y9LLP8vSyz/L0ss/y9LLP8vSyz/L0ss/y9LLP8vSyz/JUJL/yhERP8nQ0T/J0NH/y5NMP9KSjX/
				VEk7/xo/Ov8GKzn/EjxN/wojLf8NOEj/PVo7/1tgHv9cYSP/XWMn/1tgH/9bYCD/ZGoj/3yCQ/9/hUf/
				bXMs/3F3Mv99g0T/f4VH/4mOVf+IjVP/en8+/21zLP9sciv/eH07/3B2L/9QVwH/UFcB/1BXAf9QVwH/
				UFcB/1BXAf9QVwD/UFcA/1BXAP9QVwD/UFcB/1BXAf9QVwH/UFcB/1BXAf9QVwH/UFcB/1BXAf9QVwH/
				UFcB/1BXAP9QVwD/UFcA/1BXAP9QVwD/UFcA/1BXAP9QVwD/UFcA/1BXAP9QVwD/UFcA/1BXAP9QVwD/
				UFcA/1BXAP9QVwH/UFcB/1BXAf9QVwH/UFcB/1BXAP9QVwD/UFcA/1BXAP9QVwD/UFcA/09XAP9PVwD/
				UFcA/1BXAP9QVwH/UFcB/1BXAf9QVwH/UFcB/1BXAf9QVwH/UFcB/1BXAf9QVwH/UFcB/1BXAf9QVwH/
				OT9G/z5ENv86Uz7/MGRz/0F4Wf+QTBf/ukUl/yo0FP8FFh7/JIa1/wckMf8FOk7/KlZH/ztcL/87Wiv/
				O1oq/zxbKv89XC7/PFsq/zxbKv88Wyr/PFsq/zxbKv88Wyr/PFsq/zxbKv88Wyr/PFsq/zxbKv88Wyr/
				PFsq/zxbKv88Wyr/PFsq/zxbKv88Wyr/PFsq/zxbKv88Wyr/PFwq/zxcKv88XCr/PFwp/zxcKf88XCn/
				PFwq/zxcKv88XCr/PFwq/zxcKf88XCn/PFwp/zxcKf88XCn/PFwp/zxcKf88XCn/PFwp/zxcKf88XCr/
				PVsq/z1bKv89Wyr/PVsq/z1bKv89Wyr/PVsq/z1bKv89Wyr/PVsq/z1bKv89Wyr/PVsq/z1bKv89Wyr/
				PVsq/z1bKv89Wyr/PVsq/z1bKv89Wyr/PFsq/zxbKv88Wyr/PFsq/zxbKv88Wyr/PFsq/zxbKv88Wyr/
				PFsq/zxbKv88Wyr/PFsq/zxbKv86WSz/J0Rk/ydFZP8xZEX/L2ZM/zlfNf9CWSz/UFUx/yFLPf8GLDv/
				CUhi/wU6Tv8JUGv/
			</Image>
		</BitMap>
		<Widget index="0" type="434f4e53" display_nr="3" y="3" anz_rows="5" anz_cols="16">
			<Data>
				<Data>13</Data>
				<Data>7</Data>
				<Data>0</Data>
				<Data>0</Data>
			</Data>
		</Widget>
		<Widget index="1" type="44425547" display_nr="3" has_focus="true" has_scrollfocus="true" x="5" anz_rows="3" anz_cols="11">
			<Data>
				<Data>0</Data>
				<Data>0</Data>
				<Data>0</Data>
				<Data>0</Data>
			</Data>
		</Widget>
		<Widget index="2" type="53544154" display_nr="3" has_focus="true" has_scrollfocus="true" anz_rows="3" anz_cols="5" scroll_offset="6" scroll_index="6">
			<Data>
				<Data>4</Data>
				<Data>0</Data>
				<Data>0</Data>
				<Data>0</Data>
			</Data>
		</Widget>
	</View>
</MA>]]
		
		-- File configuration
		local dir = {}
		dir.main = gma.show.getvar('PATH')..'/'
		dir.importexport = dir.main..'importexport/'
		
		fn = {}
		fn.temp = [[_templ_view.xml]]
		
		file = {}
		file.temp = dir.importexport..fn.temp
		
		-- Write temp file
		local writefile = io.open(file.temp, 'w')
		writefile:write(cmd_script)
		writefile:close()
	
		-- Import and call view
		local view_slot = advanceSpace('View', 1, 1)
		gma.cmd('SelectDrive 1')
		gma.cmd('Import "'..fn.temp..'" View '..view_slot)
		gma.cmd('View '..view_slot..' /screen='..screen)
		
		-- Cleanup
		os.remove(file.temp)
		gma.cmd('Delete View '..view_slot)
	end
} --end of gui table


local function self_id(...)
	--CANNOT BE USED INSIDE MAIN RETURN FUNCTION. MUST EXECUTE ON LOAD.
	local unpack = table.unpack

	local self = select(2, ...)
	local indices = {0, 0}
	local self_id

	repeat
		indices = {self:find('%d+', indices[2] + 1)}
	until indices[2] == #self

	self_id = tonumber(self:sub(unpack(indices)))

	return self_id
end


if not tempkey.script then tempkey.script = {} end
if not tempkey.script[config.page] then tempkey.script[config.page] = {} end
tempkey.script[config.page].uninstall = self_id(...)

local completed = false
local user_interrupt = false
local uninstall_log = class.superTable:new()

local function uninstall()
	---- ACCIDENT-PROOF UNINSTALL ----
	local uninstall_confirm = gui.confirm('Confirm Temp Key Uninstallation', 
		'Are you sure you want to uninstall your Temp Page Key?',
		'Click X to Cancel')
	
	-- break installation if user does not confirm
	if not uninstall_confirm then 
		user_interrupt = true
		goto EOF
	end
	
	
	
	
	--------------------------------------
	---- BEGIN UNINSTALLATION PROCESS ----
	--------------------------------------
	
	---- REMOVE SEQUENCE AND EXECUTOR ----
	gma.cmd('Unlock Sequence '..config.seq_num)
	gma.cmd('Delete Sequence '..config.seq_num..' /nc')
	uninstall_log:append('Trigger Key and Sequence Removed')
	
	
	---- REMOVE DMX REMOTES ----
	local dmx_remotes = csv.objToArray('Remote 3.*')
	local remotes_match = class.superTable:new()
	for i, v in ipairs(dmx_remotes) do
		if v.dmx == config.remote.off then
			remotes_match:append(i)												--append index value to match-table
		end
	end
	
	
	-- Remove remote if exactly 1 match found
	if #remotes_match ~= 1 then
		uninstall_log:append('ERROR: '..#remotes_match..' of 1 remotes found for uninstall. No remotes removed.')
	else
		gma.cmd('Delete Remote 3.'..remotes_match[1])
		uninstall_log:append('DMX Remote Deleted')
	end
	
	
	---- REMOVE DUMMY FIXTURE ----
	gma.cmd('CD "EditSetup"')
	gma.cmd('CD "Layers"')
	local layers = csv.objToArray()
	for i = #layers, 1, -1 do		--traverse backwards; most likely to be an ending layer
		local layer_current = csv.objToArray(tostring(i)..'.*')
		local match = false
		for _, v in ipairs(layer_current) do		--check if fixture is in current layer
			if tonumber(v.fixid) == config.fix_id then
				match = true
				break
			end
		end
		
		
		if match then
			
			-- delete entire layer if only the one dummy fixture is located in it
			if #layer_current == 1 then
				gma.cmd('Delete '..i..' /nc')
				uninstall_log:append('No other fixtures in "'..layers[i].name..'" layer detected. Fixture and layer deleted.')
				
			-- remove only the one fixture if more exist in same layer
			else
				uninstall_log:append('Other fixtures detected in "'..layers[i].name..'" layer. Layer NOT deleted.')
				
				gma.cmd('Delete Fixture '..config.fix_id..' /nc')
				uninstall_log:append('Dummy Fixture Removed')
			end
			
			break
		end
	end

	-- return to root directory
	gma.cmd('CD /')
	
	
	
	---- Delete Plugins ----	
	for k, v in pairs(tempkey.script[config.page]) do
		gma.cmd('Unlock Plugin '..v)
		gma.cmd('Delete Plugin '..v..' /nc')
		uninstall_log:append(k:upper()..' Plugin for Temp-Page '..config.page..' deleted.')
	end
	
	-- Remove from global "registry"
	local registry_removed = false
	repeat
		local clean = true
		for i, v in ipairs(tempkey.installed) do
			--if matching page is found, it is removed, loop is repeated in case of adjacent matching values (happens during load following install, unknown reason)
			if v == config.page then
				table.remove(tempkey.installed, i)
				registry_removed = true
				clean = false
			end
		end
	until clean
	
	if registry_removed then
		uninstall_log:append('Deleted registry entry for Page '..config.page..' Key.')
	else
		uninstall_log:append('ERROR: Registry entry for Page '..config.page..' Key not found/removed.\nPlease reload plugin engine before trying to install again.')
	end
	
	

	---- USER CONFIRMATION MESSAGES ----
	-- (uninstall_log will be printed in print_log() function)
	completed = true
	
	-- Notify User
	gui.msgbox('Uninstall Complete', 'See System Monitor for Details', 'See System Monitor for Details')
	
	
::EOF::
end


local function print_log()
	uninstall_log:append('')
	if completed then uninstall_log:append('UNINSTALLER COMPLETED SUCCESSFULLY.')
	elseif user_interrupt then uninstall_log:append('Uninstall cancelled by user. No objects were altered.')
	else uninstall_log:append('Uninstaller DID NOT complete successfully. Please check System Monitor for error messages.') end
	uninstall_log:append('')
	
	for _, v in ipairs(uninstall_log) do
		gui.print(v)
	end
end

return uninstall, print_log
]===]


	createPlugin(plugins.uninstall.num, plugins.uninstall.name, pluginScript)
	gma.cmd('Lock Plugin '..plugins.uninstall.num)
	install_log:append('Uninstall plugin created. Stored at Plugin #'..plugins.uninstall.num)
	
	
	---- CONFIRM INSTALLATION INFORMATION ----
	gui.msgbox('Temp Key Installation Complete.', 'Please see System Monitor for details.', 'Please see System Monitor for details.')
	
	install_complete = true

::EOF::
end

local function print_report()
	install_log:append('')	--create space
	if not install_complete then install_log:append('Installation did not finish successfully. Please check System Monitor for error messages.')
	else install_log:append('\nInstallation completed successfully.') end
	install_log:append('')	--create space
	
	for i, v in ipairs(install_log) do
		gui.print(v)
	end
end


return install_key, print_report
